@@ -28,6 +28,10 @@ struct temp_event {
   struct temp_event *next;
 };
 
+// The following macro follows if a string function has an error. It should 
+// never happen; but it is necessary to prevent compiler warnings. We print 
+// something just in case there is programmer error in invoking the function.
+#define HANDLE_STRING_ERROR {fprintf(stderr,"%s:%i unexpected string function error.\n",__FILE__,__LINE__); exit(-1);}
 
 static CORETEMP_native_event_entry_t * _coretemp_native_events;
 static int num_events		= 0;
@@ -101,6 +105,7 @@ generateEventList(char *base_dir)
     int count = 0;
     struct dirent *hwmonx;
     int i,pathnum;
+    int retlen;
 
 #define NUM_PATHS 2
     char paths[NUM_PATHS][PATH_MAX]={
@@ -180,8 +185,13 @@ generateEventList(char *base_dir)
 	     if (fff==NULL) continue;
 	     fclose(fff);
 
-	     snprintf(name, PAPI_MAX_STR_LEN, "%s:in%i_input", 
-			 hwmonx->d_name, i);
+	     retlen = snprintf(name, PAPI_MAX_STR_LEN, "%s:in%i_input", hwmonx->d_name, i);
+	     if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+	         SUBDBG("Unable to generate name %s:in%i_input\n", hwmonx->d_name, i);
+            closedir(d);
+	         return ( PAPI_EINVAL );
+	     }
+
 	     snprintf(units, PAPI_MIN_STR_LEN, "V");
 	     snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
 		      units,modulename,
@@ -222,8 +232,14 @@ generateEventList(char *base_dir)
 	     if (fff==NULL) continue;
 	     fclose(fff);
 
-	     snprintf(name, PAPI_MAX_STR_LEN, "%s:temp%i_input", 
-			 hwmonx->d_name, i);
+	     retlen = snprintf(name, PAPI_MAX_STR_LEN, "%s:temp%i_input", hwmonx->d_name, i);
+	     if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+	         SUBDBG("Unable to generate name %s:temp%i_input\n", hwmonx->d_name, i);
+            closedir(d);
+            closedir(dir);
+	         return ( PAPI_EINVAL );
+	     }
+
 	     snprintf(units, PAPI_MIN_STR_LEN, "degrees C");
 	     snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
 		      units,modulename,
@@ -257,14 +273,26 @@ generateEventList(char *base_dir)
 	     }
 
 	     /* Look for input fan */
-	     snprintf(filename, PAPI_MAX_STR_LEN, "%s/fan%d_input", 
-		      path,i);
+	     retlen = snprintf(filename, PAPI_MAX_STR_LEN, "%s/fan%d_input", path,i);
+	     if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+	         SUBDBG("Unable to generate filename %s/fan%d_input\n", path,i);
+            closedir(d);
+            closedir(dir);
+	         return ( PAPI_EINVAL );
+	     }
+
 	     fff=fopen(filename,"r");
 	     if (fff==NULL) continue;
 	     fclose(fff);
 
-	     snprintf(name, PAPI_MAX_STR_LEN, "%s:fan%i_input", 
-			 hwmonx->d_name, i);
+	     retlen = snprintf(name, PAPI_MAX_STR_LEN, "%s:fan%i_input", hwmonx->d_name, i);
+	     if (retlen <= 0 || PAPI_MAX_STR_LEN <= retlen) {
+	         SUBDBG("Unable to generate name %s:fan%i_input\n", hwmonx->d_name, i);
+            closedir(d);
+            closedir(dir);
+	         return ( PAPI_EINVAL );
+	     }
+
 	     snprintf(units, PAPI_MIN_STR_LEN, "RPM");
 	     snprintf(description, PAPI_MAX_STR_LEN, "%s, %s module, label %s",
 		      units,modulename,
@@ -341,11 +369,12 @@ _coretemp_init_thread( hwd_context_t *ctx )
 static int
 _coretemp_init_component( int cidx )
 {
+    int retval = PAPI_OK;
      int i = 0;
      struct temp_event *t,*last;
 
      if ( is_initialized )
-	return (PAPI_OK );
+         goto fn_exit;
 
      is_initialized = 1;
 
@@ -355,37 +384,60 @@ _coretemp_init_component( int cidx )
      num_events = generateEventList("/sys/class/hwmon");
 
      if ( num_events < 0 ) {
-        strncpy(_coretemp_vector.cmp_info.disabled_reason,
-		"Cannot open /sys/class/hwmon",PAPI_MAX_STR_LEN);
-	return PAPI_ENOCMP;
+        char* strCpy;
+        strCpy=strncpy(_coretemp_vector.cmp_info.disabled_reason,
+        "Cannot open /sys/class/hwmon",PAPI_MAX_STR_LEN);
+        _coretemp_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+        retval = PAPI_ECMP;
+        goto fn_fail;
      }
 
      if ( num_events == 0 ) {
-        strncpy(_coretemp_vector.cmp_info.disabled_reason,
-		"No coretemp events found",PAPI_MAX_STR_LEN);
-	return PAPI_ENOCMP;
+        char* strCpy=strncpy(_coretemp_vector.cmp_info.disabled_reason,
+        "No coretemp events found",PAPI_MAX_STR_LEN);
+        _coretemp_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+        retval = PAPI_ECMP;
+        goto fn_fail;
      }
 
      t = root;
   
      _coretemp_native_events = (CORETEMP_native_event_entry_t*)
           papi_calloc(num_events, sizeof(CORETEMP_native_event_entry_t));
+     if (_coretemp_native_events == NULL) {
+          int strErr=snprintf(_coretemp_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "malloc() of _coretemp_native_events failed for %lu bytes.", num_events*sizeof(CORETEMP_native_event_entry_t));
+          _coretemp_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+          if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+          retval = PAPI_ENOMEM;
+          goto fn_fail;
+     }
 
      do {
-	strncpy(_coretemp_native_events[i].name,t->name,PAPI_MAX_STR_LEN);
+        char *strCpy;
+        strCpy=strncpy(_coretemp_native_events[i].name,t->name,PAPI_MAX_STR_LEN-2);
         _coretemp_native_events[i].name[PAPI_MAX_STR_LEN-1] = '\0';
-	strncpy(_coretemp_native_events[i].path,t->path,PATH_MAX);
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+
+        strCpy=strncpy(_coretemp_native_events[i].path,t->path,PATH_MAX);
         _coretemp_native_events[i].path[PATH_MAX-1] = '\0';
-	strncpy(_coretemp_native_events[i].units,t->units,PAPI_MIN_STR_LEN);
-	_coretemp_native_events[i].units[PAPI_MIN_STR_LEN-1] = '\0';
-	strncpy(_coretemp_native_events[i].description,t->description,PAPI_MAX_STR_LEN);
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+
+        strCpy=strncpy(_coretemp_native_events[i].units,t->units,PAPI_MIN_STR_LEN-2);
+	    _coretemp_native_events[i].units[PAPI_MIN_STR_LEN-1] = '\0';
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+
+        strCpy=strncpy(_coretemp_native_events[i].description,t->description,PAPI_MAX_STR_LEN-2);
         _coretemp_native_events[i].description[PAPI_MAX_STR_LEN-1] = '\0';
-	_coretemp_native_events[i].stone = 0;
-	_coretemp_native_events[i].resources.selector = i + 1;
-	last	= t;
-	t		= t->next;
-	papi_free(last);
-	i++;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+
+	    _coretemp_native_events[i].stone = 0;
+	    _coretemp_native_events[i].resources.selector = i + 1;
+	    last	= t;
+	    t		= t->next;
+	    papi_free(last);
+	    i++;
      } while (t != NULL);
      root = NULL;
 
@@ -395,7 +447,11 @@ _coretemp_init_component( int cidx )
      /* Export the component id */
      _coretemp_vector.cmp_info.CmpIdx = cidx;
 
-     return PAPI_OK;
+  fn_exit:
+     _papi_hwd[cidx]->cmp_info.disabled = retval;
+     return retval;
+  fn_fail:
+     goto fn_exit;
 }
 
 
