@@ -117,6 +117,9 @@ generateNetEventList( void )
     if (fin == NULL) {
         SUBDBG("Can't find %s, are you sure the /proc file-system is mounted?\n",
            NET_PROC_FILE);
+        snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+        "Failed to find /proc file-system.");
+        _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
         return 0;
     }
 
@@ -126,6 +129,9 @@ generateNetEventList( void )
         if (retval == NULL) {
         	fclose(fin);
             SUBDBG("Not enough lines in %s\n", NET_PROC_FILE);
+            snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+            "Not enough lines in '%s'.", NET_PROC_FILE);
+            _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
             return 0;
         }
     }
@@ -150,6 +156,9 @@ generateNetEventList( void )
             if (!temp) {
                 PAPIERROR("out of memory!");
                 fclose(fin);
+                snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                "%s failed to allocate %lu bytes for temp_event.", __func__, sizeof(struct temp_event));
+                _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
                 return PAPI_ENOMEM;
             }
             temp->next = NULL;
@@ -162,6 +171,9 @@ generateNetEventList( void )
                 free(temp);
                 fclose(fin);
                 PAPIERROR("This shouldn't be possible\n");
+                snprintf(_net_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                "%s Error navigating linked list.", __func__);
+                _net_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
                 return PAPI_ECMP;
             }
             last = temp;
@@ -298,11 +310,12 @@ _net_init_thread( hwd_context_t *ctx )
 static int
 _net_init_component( int cidx  )
 {
+    int retval = PAPI_OK;
     int i = 0;
     struct temp_event *t, *last;
 
     if ( is_initialized )
-        return PAPI_OK;
+        goto fn_exit;
 
     memset(_net_register_start, 0,
     		NET_MAX_COUNTERS*sizeof(_net_register_start[0]));
@@ -314,11 +327,13 @@ _net_init_component( int cidx  )
     /* The network interfaces are listed in /proc/net/dev */
     num_events = generateNetEventList();
 
-    if ( num_events < 0 )  /* PAPI errors */
-        return num_events;
+    if ( num_events < 0 ) {  /* disabled_reason already set */
+        retval = PAPI_ECMP;
+        goto fn_fail;
+    }
 
     if ( num_events == 0 )  /* No network interfaces found */
-        return PAPI_OK;
+        goto fn_exit;
 
     t = root;
     _net_native_events = (NET_native_event_entry_t*)
@@ -342,7 +357,11 @@ _net_init_component( int cidx  )
     /* Export the component id */
     _net_vector.cmp_info.CmpIdx = cidx;
 
-    return PAPI_OK;
+  fn_exit:
+    _papi_hwd[cidx]->cmp_info.disabled = retval;
+    return retval;
+  fn_fail:
+    goto fn_exit;
 }
 
 
@@ -564,7 +583,7 @@ _net_ntv_enum_events( unsigned int *EventCode, int modifier )
  *
  */
 static int
-_net_ntv_name_to_code( char *name, unsigned int *EventCode )
+_net_ntv_name_to_code( const char *name, unsigned int *EventCode )
 {
     int i;
 
