@@ -129,6 +129,10 @@ static void* dl1 = NULL;
 static void* dl2 = NULL;
 static void* dl3 = NULL;
 
+static char cuda_main[]=PAPI_CUDA_MAIN;
+static char cuda_runtime[]=PAPI_CUDA_RUNTIME;
+static char nvml_main[]=PAPI_NVML_MAIN;
+
 static int linkCudaLibraries();
 
 /* Declare our vector in advance */
@@ -175,6 +179,7 @@ getClockSpeed(nvmlDevice_t dev, nvmlClockType_t which_one)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
 
     return (unsigned long long)ret;
@@ -190,6 +195,7 @@ getEccLocalErrors(nvmlDevice_t dev, nvmlEccBitType_t bits, int which_one)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
     switch (which_one) {
     case LOCAL_ECC_REGFILE:
@@ -215,6 +221,7 @@ getFanSpeed(nvmlDevice_t dev)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
     return (unsigned long long)ret;
 }
@@ -228,6 +235,7 @@ getMaxClockSpeed(nvmlDevice_t dev, nvmlClockType_t which_one)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
     return (unsigned long long) ret;
 }
@@ -241,6 +249,7 @@ getMemoryInfo(nvmlDevice_t dev, int which_one)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
 
     switch (which_one) {
@@ -266,40 +275,57 @@ getPState(nvmlDevice_t dev)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
     switch (state) {
     case NVML_PSTATE_15:
         ret++;
+        // fall through
     case NVML_PSTATE_14:
         ret++;
+        // fall through
     case NVML_PSTATE_13:
         ret++;
+        // fall through
     case NVML_PSTATE_12:
         ret++;
+        // fall through
     case NVML_PSTATE_11:
         ret++;
+        // fall through
     case NVML_PSTATE_10:
         ret++;
+        // fall through
     case NVML_PSTATE_9:
         ret++;
+        // fall through
     case NVML_PSTATE_8:
         ret++;
+        // fall through
     case NVML_PSTATE_7:
         ret++;
+        // fall through
     case NVML_PSTATE_6:
         ret++;
+        // fall through
     case NVML_PSTATE_5:
         ret++;
+        // fall through
     case NVML_PSTATE_4:
         ret++;
+        // fall through
     case NVML_PSTATE_3:
         ret++;
+        // fall through
     case NVML_PSTATE_2:
         ret++;
+        // fall through
     case NVML_PSTATE_1:
         ret++;
+        // fall through
     case NVML_PSTATE_0:
         break;
+        // fall through
     case NVML_PSTATE_UNKNOWN:
     default:
         /* This should never happen?
@@ -318,6 +344,7 @@ getPowerUsage(nvmlDevice_t dev)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
     return (unsigned long long) power;
 }
@@ -331,6 +358,7 @@ getTemperature(nvmlDevice_t dev)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
     return (unsigned long long)ret;
 }
@@ -344,6 +372,7 @@ getTotalEccErrors(nvmlDevice_t dev, nvmlEccBitType_t bits)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
     return counts;
 }
@@ -360,6 +389,7 @@ getUtilization(nvmlDevice_t dev, int which_one)
 
     if (NVML_SUCCESS != bad) {
         SUBDBG("something went wrong %s\n", (*nvmlErrorStringPtr)(bad));
+        return (unsigned long long) - 1;
     }
 
     switch (which_one) {
@@ -386,6 +416,24 @@ unsigned long long getPowerManagementLimit(nvmlDevice_t dev)
     return (unsigned long long) limit;
 }
 
+static int _papi_nvml_init_private(void);
+
+/*
+ * Check for the initialization step and does it if needed
+ */
+static int
+_nvml_check_n_initialize(papi_vector_t *vector)
+{
+  if (!vector->cmp_info.initialized)
+      return _papi_nvml_init_private();
+  return PAPI_OK;
+}
+
+#define DO_SOME_CHECKING(vectorp) do {           \
+  int err = _nvml_check_n_initialize(vectorp);   \
+  if (PAPI_OK != err) return err;                \
+} while(0)
+
 static void
 nvml_hardware_reset()
 {
@@ -493,6 +541,8 @@ nvml_hardware_read(long long *value, int which_one)
     default:
         return PAPI_EINVAL;
     }
+    if (*value == (long long)(unsigned long long) - 1)
+        return PAPI_EINVAL;
 
     return PAPI_OK;
 }
@@ -564,19 +614,16 @@ static int
 detectDevices()
 {
     nvmlReturn_t ret;
-    nvmlEnableState_t mode = NVML_FEATURE_DISABLED;
+    nvmlEnableState_t mode        = NVML_FEATURE_DISABLED;
+    nvmlEnableState_t pendingmode = NVML_FEATURE_DISABLED;
 
     char name[64];
     char inforomECC[16];
-    char inforomPower[16];
     char names[device_count][64];
 
     float ecc_version = 0.0;
-    float power_version = 0.0;
 
     int i = 0;
-    int isTesla = 0;
-    int isFermi = 0;
 
     unsigned int temp = 0;
 
@@ -584,8 +631,6 @@ detectDevices()
 
     /* So for each card, check whats querable */
     for (i = 0; i < device_count; i++) {
-        isTesla = 0;
-        isFermi = 1;
         features[i] = 0;
         
         ret = (*nvmlDeviceGetHandleByIndexPtr)(i, &devices[i]);
@@ -605,23 +650,11 @@ detectDevices()
         ret = (*nvmlDeviceGetInforomVersionPtr)(devices[i], NVML_INFOROM_ECC, inforomECC, 16);
         if (NVML_SUCCESS != ret) {
             SUBDBG("nvmlGetInforomVersion fails %s\n", (*nvmlErrorStringPtr)(ret));
-            isFermi = 0;
-        }
-        ret = (*nvmlDeviceGetInforomVersionPtr)(devices[i], NVML_INFOROM_POWER, inforomPower, 16);
-        if (NVML_SUCCESS != ret) {
-            /* This implies the card is older then Fermi */
-            SUBDBG("nvmlGetInforomVersion fails %s\n", (*nvmlErrorStringPtr)(ret));
-            SUBDBG("Based upon the return to nvmlGetInforomVersion, we conclude this card is older then Fermi.\n");
-            isFermi = 0;
+        } else {
+            ecc_version = strtof(inforomECC, NULL);
         }
 
-        ecc_version = strtof(inforomECC, NULL);
-        power_version = strtof(inforomPower, NULL);
-
-        isTesla = (NULL == strstr(name, "Tesla")) ? 0 : 1;
-
-        /* For Tesla and Quadro products from Fermi and Kepler families. */
-        if (isFermi) {
+        if (getClockSpeed(devices[i], NVML_CLOCK_GRAPHICS) != (unsigned long long) - 1) {
             features[i] |= FEATURE_CLOCK_INFO;
             num_events += 3;
         }
@@ -630,7 +663,7 @@ detectDevices()
             requires NVML_INFOROM_ECC 2.0 or higher for location-based counts
             requires NVML_INFOROM_ECC 1.0 or higher for all other ECC counts
             requires ECC mode to be enabled. */
-        ret = (*nvmlDeviceGetEccModePtr)(devices[i], &mode, NULL);
+        ret = (*nvmlDeviceGetEccModePtr)(devices[i], &mode, &pendingmode);
         if (NVML_SUCCESS == ret) {
             if (NVML_FEATURE_ENABLED == mode) {
                 if (ecc_version >= 2.0) {
@@ -646,12 +679,14 @@ detectDevices()
             SUBDBG("nvmlDeviceGetEccMode does not appear to be supported. (nvml return code %d)\n", ret);
         }
 
-        /* For all discrete products with dedicated fans */
-        features[i] |= FEATURE_FAN_SPEED;
-        num_events++;
+        /* Check if fan speed is available */
+        if (getFanSpeed(devices[i]) != (unsigned long long) - 1) {
+            features[i] |= FEATURE_FAN_SPEED;
+            num_events++;
+        }
 
-        /* For Tesla and Quadro products from Fermi and Kepler families. */
-        if (isFermi) {
+        /* Check if clock data are available */
+        if (getMaxClockSpeed(devices[i], NVML_CLOCK_GRAPHICS) != (unsigned long long) - 1) {
             features[i] |= FEATURE_MAX_CLOCK;
             num_events += 3;
         }
@@ -660,8 +695,8 @@ detectDevices()
         features[i] |= FEATURE_MEMORY_INFO;
         num_events += 3; /* total, free, used */
 
-        /* For Tesla and Quadro products from the Fermi and Kepler families. */
-        if (isFermi) {
+        /* Check if performance state is available */
+        if (getPState(devices[i]) != (unsigned long long) - 1) {
             features[i] |= FEATURE_PERF_STATES;
             num_events++;
         }
@@ -679,9 +714,11 @@ detectDevices()
             SUBDBG("nvmlDeviceGetPowerUsage does not appear to be supported on this card. (nvml return code %d)\n", ret);
         }
 
-        /* For all discrete and S-class products. */
-        features[i] |= FEATURE_TEMP;
-        num_events++;
+        /* Check if temperature data are available */
+        if (getTemperature(devices[i]) != (unsigned long long) - 1) {
+            features[i] |= FEATURE_TEMP;
+            num_events++;
+        }
 
         // For power_management_limit
         {
@@ -716,8 +753,8 @@ detectDevices()
             SUBDBG("Done nvmlDeviceGetPowerManagementLimitConstraintsPtr\n");
         }
 
-        /* For Tesla and Quadro products from the Fermi and Kepler families */
-        if (isFermi) {
+        /* Check if temperature data are available */
+        if (getUtilization(devices[i], GPU_UTILIZATION) != (unsigned long long) - 1) {
             features[i] |= FEATURE_UTILIZATION;
             num_events += 2;
         }
@@ -995,59 +1032,124 @@ createNativeEvents()
         strncpy(names[i], name, sizeof(names[0]) - 1);
         names[i][sizeof(names[0]) - 1] = '\0';
     }
+} // create native events.
+
+
+// Triggered by PAPI_shutdown(), but also if init fails to complete; for example due
+// to a missing library. We still need to clean up. The dynamic libs (dlxxx routines)
+// may have open mallocs that need to be free()d.
+ 
+int _papi_nvml_shutdown_component()
+{
+    SUBDBG("Enter:\n");
+    nvml_hardware_reset();
+    if (nvml_native_table != NULL) papi_free(nvml_native_table);
+    if (devices != NULL) papi_free(devices);
+    if (features != NULL) papi_free(features);
+    if (power_management_initial_limit) papi_free(power_management_initial_limit);
+    if (power_management_limit_constraint_min) papi_free(power_management_limit_constraint_min);
+    if (power_management_limit_constraint_max) papi_free(power_management_limit_constraint_max);
+    if (nvmlShutdownPtr) (*nvmlShutdownPtr)();        // Call nvml shutdown if we got that far.
+
+    device_count = 0;
+    num_events = 0;
+
+    // close the dynamic libraries needed by this component (opened in the init component call)
+    if (dl3) {dlclose(dl3); dl3=NULL;}
+    if (dl2) {dlclose(dl2); dl2=NULL;}
+    if (dl1) {dlclose(dl1); dl1=NULL;}
+
+    return PAPI_OK;
 }
 
+
+
 /** Initialize hardware counters, setup the function vector table
  * and get hardware information, this routine is called when the
  * PAPI process is initialized (IE PAPI_library_init)
  */
-int
-_papi_nvml_init_component(int cidx)
+
+static int _papi_nvml_init_component(int cidx)
 {
     SUBDBG("Entry: cidx: %d\n", cidx);
+    /* Export the total number of events available */
+    _nvml_vector.cmp_info.num_native_events = -1;
+
+    /* Export the component id */
+    _nvml_vector.cmp_info.CmpIdx = cidx;
+
+    /* Export the number of 'counters' */
+    _nvml_vector.cmp_info.num_cntrs = -1;
+    _nvml_vector.cmp_info.num_mpx_cntrs = -1;
+
+     sprintf(_nvml_vector.cmp_info.disabled_reason,
+             "Not initialized. Access component events to initialize it.");
+    _nvml_vector.cmp_info.disabled = PAPI_EDELAY_INIT;
+
+    return PAPI_EDELAY_INIT;
+}
+
+int _papi_nvml_init_private(void)
+{
     nvmlReturn_t ret;
     cudaError_t cuerr;
-    int papi_errorcode;
+    int err = PAPI_OK;
 
     int cuda_count = 0;
     unsigned int nvml_count = 0;
 
+    PAPI_lock(COMPONENT_LOCK);
+    if (_nvml_vector.cmp_info.initialized) goto nvml_init_private_exit;
+
+    SUBDBG("Private init with component idx: %d\n", _nvml_vector.cmp_info.CmpIdx);
     /* link in the cuda and nvml libraries and resolve the symbols we need to use */
     if (linkCudaLibraries() != PAPI_OK) {
         SUBDBG("Dynamic link of CUDA libraries failed, component will be disabled.\n");
         SUBDBG("See disable reason in papi_component_avail output for more details.\n");
-        return (PAPI_ENOSUPP);
+        _papi_nvml_shutdown_component();                          // clean up any open dynLibs, mallocs, etc.
+        err = (PAPI_ENOSUPP);
+        goto nvml_init_private_exit;
     }
 
     ret = (*nvmlInitPtr)();
     if (NVML_SUCCESS != ret) {
-        strcpy(_nvml_vector.cmp_info.disabled_reason, "The NVIDIA managament library failed to initialize.");
-        return PAPI_ENOSUPP;
+        strcpy(_nvml_vector.cmp_info.disabled_reason, "The NVIDIA management library failed to initialize.");
+        _papi_nvml_shutdown_component();                          // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOSUPP;
+        goto nvml_init_private_exit;
     }
 
     cuerr = (*cuInitPtr)(0);
     if (cudaSuccess != cuerr) {
         strcpy(_nvml_vector.cmp_info.disabled_reason, "The CUDA library failed to initialize.");
-        return PAPI_ENOSUPP;
+        _papi_nvml_shutdown_component();                          // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOSUPP;
+        goto nvml_init_private_exit;
     }
 
     /* Figure out the number of CUDA devices in the system */
     ret = (*nvmlDeviceGetCountPtr)(&nvml_count);
     if (NVML_SUCCESS != ret) {
-        strcpy(_nvml_vector.cmp_info.disabled_reason, "Unable to get a count of devices from the NVIDIA managament library.");
-        return PAPI_ENOSUPP;
+        strcpy(_nvml_vector.cmp_info.disabled_reason, "Unable to get a count of devices from the NVIDIA management library.");
+        _papi_nvml_shutdown_component();                          // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOSUPP;
+        goto nvml_init_private_exit;
     }
 
     cuerr = (*cudaGetDeviceCountPtr)(&cuda_count);
     if (cudaSuccess != cuerr) {
         strcpy(_nvml_vector.cmp_info.disabled_reason, "Unable to get a device count from CUDA.");
-        return PAPI_ENOSUPP;
+        _papi_nvml_shutdown_component();                          // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOSUPP;
+        goto nvml_init_private_exit;
     }
 
     /* We can probably recover from this, when we're clever */
     if ((cuda_count > 0) && (nvml_count != (unsigned int)cuda_count)) {
-        strcpy(_nvml_vector.cmp_info.disabled_reason, "CUDA and the NVIDIA managament library have different device counts.");
-        return PAPI_ENOSUPP;
+        strcpy(_nvml_vector.cmp_info.disabled_reason, "CUDA and the NVIDIA management library have different device counts.");
+        _papi_nvml_shutdown_component();                          // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOSUPP;
+        goto nvml_init_private_exit;
     }
 
     device_count = cuda_count;
@@ -1055,21 +1157,61 @@ _papi_nvml_init_component(int cidx)
 
     /* A per device representation of what events are present */
     features = (int*)papi_malloc(sizeof(int) * device_count);
+    if (features == NULL) { 
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                    "%s failed to alloc %lu bytes for features.", __func__, sizeof(int)*device_count);
+                    _nvml_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
+        _papi_nvml_shutdown_component();                        // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOMEM;
+        goto nvml_init_private_exit;
+    }
 
     /* Handles to each device */
     devices = (nvmlDevice_t*)papi_malloc(sizeof(nvmlDevice_t) * device_count);
+    if (devices == NULL) {
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                    "%s failed to alloc %lu bytes for features.", __func__, (sizeof(nvmlDevice_t) * device_count));
+                    _nvml_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
+        _papi_nvml_shutdown_component();                        // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOMEM;
+        goto nvml_init_private_exit;
+    }
 
     /* For each device, store the intial power value to enable reset if power is altered */
     power_management_initial_limit = (unsigned int*)papi_malloc(sizeof(unsigned int) * device_count);
+    if (power_management_initial_limit == NULL) {
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                    "%s failed to alloc %lu bytes for power_management_initial_limit.", __func__, (sizeof(unsigned int) * device_count));
+                    _nvml_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
+        _papi_nvml_shutdown_component();                        // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOMEM;
+        goto nvml_init_private_exit;
+    }
     power_management_limit_constraint_min = (unsigned int*)papi_malloc(sizeof(unsigned int) * device_count);
+    if (power_management_limit_constraint_min == NULL) {
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                    "%s failed to alloc %lu bytes for power_management_limit_constraint_min.", __func__, (sizeof(unsigned int) * device_count));
+                    _nvml_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
+        _papi_nvml_shutdown_component();                        // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOMEM;
+        goto nvml_init_private_exit;
+    }
     power_management_limit_constraint_max = (unsigned int*)papi_malloc(sizeof(unsigned int) * device_count);
+    if (power_management_limit_constraint_max == NULL) {
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN-2,
+                    "%s failed to alloc %lu bytes for power_management_limit_constraint_max.", __func__, (sizeof(unsigned int) * device_count));
+                    _nvml_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;    // force null termination.
+        _papi_nvml_shutdown_component();                        // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOMEM;
+        goto nvml_init_private_exit;
+    }
 
     /* Figure out what events are supported on each card. */
-    if ((papi_errorcode = detectDevices()) != PAPI_OK) {
-        papi_free(features);
-        papi_free(devices);
+    if (detectDevices() != PAPI_OK) {
         sprintf(_nvml_vector.cmp_info.disabled_reason, "An error occured in device feature detection, please check your NVIDIA Management Library and CUDA install.");
-        return PAPI_ENOSUPP;
+        _papi_nvml_shutdown_component();                        // clean up any open dynLibs, mallocs, etc.
+        err = PAPI_ENOSUPP;
+        goto nvml_init_private_exit;
     }
 
     /* The assumption is that if everything went swimmingly in detectDevices,
@@ -1079,14 +1221,17 @@ _papi_nvml_init_component(int cidx)
     /* Export the total number of events available */
     _nvml_vector.cmp_info.num_native_events = num_events;
 
-    /* Export the component id */
-    _nvml_vector.cmp_info.CmpIdx = cidx;
-
     /* Export the number of 'counters' */
     _nvml_vector.cmp_info.num_cntrs = num_events;
     _nvml_vector.cmp_info.num_mpx_cntrs = num_events;
 
-    return PAPI_OK;
+nvml_init_private_exit:
+    _nvml_vector.cmp_info.initialized = 1;
+    _nvml_vector.cmp_info.disabled = err;
+
+    PAPI_unlock(COMPONENT_LOCK);
+
+    return err;
 }
 
 /*
@@ -1098,30 +1243,84 @@ _papi_nvml_init_component(int cidx)
 static int
 linkCudaLibraries()
 {
+    char path_lib[1024];
     /* Attempt to guess if we were statically linked to libc, if so bail */
     if (_dl_non_dynamic_init != NULL) {
         strncpy(_nvml_vector.cmp_info.disabled_reason, "NVML component does not support statically linking of libc.", PAPI_MAX_STR_LEN);
         return PAPI_ENOSUPP;
     }
 
-    /* Need to link in the cuda libraries, if not found disable the component */
-    dl1 = dlopen("libcuda.so", RTLD_NOW | RTLD_GLOBAL);
-    if (!dl1) {
-        strncpy(_nvml_vector.cmp_info.disabled_reason, "CUDA library libcuda.so not found.", PAPI_MAX_STR_LEN);
-        return (PAPI_ENOSUPP);
+    // Need to link in the cuda libraries, if any not found disable the component.
+    // getenv returns NULL if environment variable is not found.
+    char *cuda_root = getenv("PAPI_CUDA_ROOT");
+    dl1 = NULL;                                                 // Ensure reset to NULL.
+
+    // Step 1: Process override if given.   
+    if (strlen(cuda_main) > 0) {                                // If override given, it has to work.
+        dl1 = dlopen(cuda_main, RTLD_NOW | RTLD_GLOBAL);        // Try to open that path.
+        if (dl1 == NULL) {
+            snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "PAPI_CUDA_MAIN override '%s' given in Rules.nvml not found.", cuda_main);
+            return(PAPI_ENOSUPP);   // Override given but not found.
+        }
     }
+
+    // Step 2: Try system paths, will work with Spack, LD_LIBRARY_PATH, default paths.
+    if (dl1 == NULL) {                                          // No override,
+        dl1 = dlopen("libcuda.so", RTLD_NOW | RTLD_GLOBAL);     // Try system paths.
+    }
+
+    // Step 3: Try the explicit install default. 
+    if (dl1 == NULL && cuda_root != NULL) {                          // if root given, try it.
+        snprintf(path_lib, 1024, "%s/lib64/libcuda.so", cuda_root);  // PAPI Root check.
+        dl1 = dlopen(path_lib, RTLD_NOW | RTLD_GLOBAL);              // Try to open that path.
+    }
+
+    // Check for failure.
+    if (dl1 == NULL) {
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "libcuda.so not found.");
+        return(PAPI_ENOSUPP);
+    }
+
+    // We have a dl1. (libcuda.so).
+
     cuInitPtr = dlsym(dl1, "cuInit");
     if (dlerror() != NULL) {
         strncpy(_nvml_vector.cmp_info.disabled_reason, "CUDA function cuInit not found.", PAPI_MAX_STR_LEN);
         return (PAPI_ENOSUPP);
     }
 
-    dl2 = dlopen("libcudart.so", RTLD_NOW | RTLD_GLOBAL | RTLD_NODELETE);
-    if (!dl2) {
-        strncpy(_nvml_vector.cmp_info.disabled_reason, "CUDA runtime library libcudart.so not found.", PAPI_MAX_STR_LEN);
-        return (PAPI_ENOSUPP);
+    /* Need to link in the cuda runtime library, if not found disable the component */
+    dl2 = NULL;                                 // Ensure reset to NULL.
+
+    // Step 1: Process override if given.   
+    if (strlen(cuda_runtime) > 0) {                                // If override given, it has to work.
+        dl2 = dlopen(cuda_runtime, RTLD_NOW | RTLD_GLOBAL);        // Try to open that path.
+        if (dl2 == NULL) {
+            snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "PAPI_CUDA_RUNTIME override '%s' given in Rules.nvml not found.", cuda_runtime);
+            return(PAPI_ENOSUPP);   // Override given but not found.
+        }
     }
-    cudaGetDevicePtr = dlsym(dl2, "cudaGetDevice");
+
+    // Step 2: Try system paths, will work with Spack, LD_LIBRARY_PATH, default paths.
+    if (dl2 == NULL) {                                          // No override,
+        dl2 = dlopen("libcudart.so", RTLD_NOW | RTLD_GLOBAL);   // Try system paths.
+    }
+
+    // Step 3: Try the explicit install default. 
+    if (dl2 == NULL && cuda_root != NULL) {                             // if root given, try it.
+        snprintf(path_lib, 1024, "%s/lib64/libcudart.so", cuda_root);   // PAPI Root check.
+        dl2 = dlopen(path_lib, RTLD_NOW | RTLD_GLOBAL);                 // Try to open that path.
+    }
+
+    // Check for failure.
+    if (dl2 == NULL) {
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "libcudart.so not found.");
+        return(PAPI_ENOSUPP);
+    }
+
+    // We have a dl2. (libcudart.so).
+    
+   cudaGetDevicePtr = dlsym(dl2, "cudaGetDevice");
     if (dlerror() != NULL) {
         strncpy(_nvml_vector.cmp_info.disabled_reason, "CUDART function cudaGetDevice not found.", PAPI_MAX_STR_LEN);
         return (PAPI_ENOSUPP);
@@ -1137,11 +1336,37 @@ linkCudaLibraries()
         return (PAPI_ENOSUPP);
     }
 
-    dl3 = dlopen("libnvidia-ml.so", RTLD_NOW | RTLD_GLOBAL);
-    if (!dl3) {
-        strncpy(_nvml_vector.cmp_info.disabled_reason, "NVML runtime library libnvidia-ml.so not found.", PAPI_MAX_STR_LEN);
-        return (PAPI_ENOSUPP);
+    // We need the NVML main library, normally libnvidia-ml.so. 
+    dl3 = NULL;                                                 // Ensure reset to NULL.
+
+    // Step 1: Process override if given.   
+    if (strlen(nvml_main) > 0) {                                        // If override given, it MUST work.
+        dl3 = dlopen(nvml_main, RTLD_NOW | RTLD_GLOBAL);                // Try to open that path.
+        if (dl3 == NULL) {
+            snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "PAPI_NVML_MAIN override '%s' given in Rules.nvml not found.", nvml_main);
+            return(PAPI_ENOSUPP);   // Override given but not found.
+        }
     }
+
+    // Step 2: Try system paths, will work with Spack, LD_LIBRARY_PATH, default paths.
+    if (dl3 == NULL) {                                              // If no override,
+        dl3 = dlopen("libnvidia-ml.so", RTLD_NOW | RTLD_GLOBAL);    // Try system paths.
+    }
+
+    // Step 3: Try the explicit install default. 
+    if (dl3 == NULL && cuda_root != NULL) {                                         // If ROOT given, it doesn't HAVE to work.
+        snprintf(path_lib, 1024, "%s/lib64/libnvidia-ml.so", cuda_root);            // PAPI Root check.
+        dl3 = dlopen(path_lib, RTLD_NOW | RTLD_GLOBAL);                             // Try to open that path.
+    }
+
+    // Check for failure.
+    if (dl3 == NULL) {
+        snprintf(_nvml_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN, "libnvidia-ml.so not found.");
+        return(PAPI_ENOSUPP);   // Not found on default paths.
+    }
+
+    // We have a dl3. (libnvidia-ml.so).
+
     nvmlDeviceGetClockInfoPtr = dlsym(dl3, "nvmlDeviceGetClockInfo");
     if (dlerror() != NULL) {
         strncpy(_nvml_vector.cmp_info.disabled_reason, "NVML function nvmlDeviceGetClockInfo not found.", PAPI_MAX_STR_LEN);
@@ -1259,6 +1484,7 @@ int
 _papi_nvml_init_control_state(hwd_control_state_t * ctl)
 {
     SUBDBG("nvml_init_control_state... %p\n", ctl);
+    DO_SOME_CHECKING(&_nvml_vector);
     nvml_control_state_t *nvml_ctl = (nvml_control_state_t *) ctl;
     memset(nvml_ctl, 0, sizeof(nvml_control_state_t));
 
@@ -1278,6 +1504,7 @@ _papi_nvml_update_control_state(hwd_control_state_t *ctl,
     nvml_control_state_t *nvml_ctl = (nvml_control_state_t *) ctl;
     (void) ctx;
 
+    DO_SOME_CHECKING(&_nvml_vector);
     /* if no events, return */
     if (count == 0) return PAPI_OK;
 
@@ -1399,31 +1626,6 @@ _papi_nvml_reset(hwd_context_t * ctx, hwd_control_state_t * ctl)
     return PAPI_OK;
 }
 
-/** Triggered by PAPI_shutdown() */
-int
-_papi_nvml_shutdown_component()
-{
-    SUBDBG("Enter:\n");
-    nvml_hardware_reset();
-    if (nvml_native_table != NULL) papi_free(nvml_native_table);
-    if (devices != NULL) papi_free(devices);
-    if (features != NULL) papi_free(features);
-    if (power_management_initial_limit) papi_free(power_management_initial_limit);
-    if (power_management_limit_constraint_min) papi_free(power_management_limit_constraint_min);
-    if (power_management_limit_constraint_max) papi_free(power_management_limit_constraint_max);
-   (*nvmlShutdownPtr)();
-
-    device_count = 0;
-    num_events = 0;
-
-    // close the dynamic libraries needed by this component (opened in the init component call)
-    if (dl3) dlclose(dl3); dl3=NULL;
-    if (dl2) dlclose(dl2); dl2=NULL;
-    if (dl1) dlclose(dl1); dl1=NULL;
-
-    return PAPI_OK;
-}
-
 /** Called at thread shutdown */
 int
 _papi_nvml_shutdown_thread(hwd_context_t *ctx)
@@ -1510,6 +1712,8 @@ _papi_nvml_ntv_enum_events(unsigned int *EventCode, int modifier)
 {
     int index;
 
+    DO_SOME_CHECKING(&_nvml_vector);
+
     switch (modifier) {
 
     /* return EventCode of first event */
@@ -1552,6 +1756,8 @@ _papi_nvml_ntv_code_to_name(unsigned int EventCode, char *name, int len)
     SUBDBG("Entry: EventCode: %#x, name: %s, len: %d\n", EventCode, name, len);
     int index;
 
+    DO_SOME_CHECKING(&_nvml_vector);
+
     index = EventCode;
 
     /* Make sure we are in range */
@@ -1641,6 +1847,7 @@ papi_vector_t _nvml_vector = {
         .cntr_umasks = 0,
         .cpu = 0,
         .inherit = 0,
+        .initialized = 0,
     },
 
     /* sizes of framework-opaque component-private structures */
