@@ -64,6 +64,11 @@
 #define PERF_EVENTS_OPENED  0x01
 #define PERF_EVENTS_RUNNING 0x02
 
+// The following macro follows if a string function has an error. It should 
+// never happen; but it is necessary to prevent compiler warnings. We print 
+// something just in case there is programmer error in invoking the function.
+#define HANDLE_STRING_ERROR {fprintf(stderr,"%s:%i unexpected string function error.\n",__FILE__,__LINE__); exit(-1);}
+
 /* Forward declaration */
 papi_vector_t _perf_event_vector;
 
@@ -132,7 +137,8 @@ pe_vendor_fixups(papi_vector_t *vector)
   }
 
 	/* ARM */
-	if ( _papi_hwi_system_info.hw_info.vendor == PAPI_VENDOR_ARM) {
+	/* If implementer is ARM Limited. */
+	if ( _papi_hwi_system_info.hw_info.vendor == PAPI_VENDOR_ARM_ARM) {
 
 		/* Some ARMv7 and earlier could not measure	*/
 		/* KERNEL and USER separately.			*/
@@ -268,7 +274,7 @@ get_read_format( unsigned int multiplex,
 /* attr.exclude_guest is enabled by default in recent libpfm4 */
 /* however older kernels will reject events with it set */
 /* because the reserved field is not all zeros */
-static int
+void
 check_exclude_guest( void )
 {
 	int ev_fd;
@@ -283,7 +289,7 @@ check_exclude_guest( void )
 	ev_fd = sys_perf_event_open( &attr, 0, -1, -1, 0 );
 	if ( ev_fd == -1 ) {
 		PAPIERROR("Couldn't open hw_instructions in exclude_guest=0 test");
-		return -1;
+		return;
 	}
 	close(ev_fd);
 
@@ -305,7 +311,7 @@ check_exclude_guest( void )
 		close(ev_fd);
 	}
 
-	return PAPI_OK;
+	return;
 }
 
 /*****************************************************************/
@@ -398,6 +404,7 @@ static int map_perf_event_errors_to_papi(int perf_event_error) {
       case ENOENT:
 	   ret = PAPI_ENOEVNT;
            break;
+	case ESRCH:	/* If cannnot find process to attach to */
       case ENOSYS:
       case EAGAIN:
       case EBUSY:
@@ -683,11 +690,23 @@ open_pe_events( pe_context_t *ctx, pe_control_t *ctl )
 	int i, ret = PAPI_OK;
 	long pid;
 
-	if (ctl->granularity==PAPI_GRN_SYS) {
-		pid = -1;
+
+	/* Set the pid setting */
+	/* If attached, this is the pid of process we are attached to. */
+	/* If GRN_THRD then it is 0 meaning current process only */
+	/* If GRN_SYS then it is -1 meaning all procs on this CPU */
+	/* Note if GRN_SYS then CPU must be specified, not -1 */
+
+	if (ctl->attached) {
+		pid = ctl->tid;
 	}
 	else {
-		pid = ctl->tid;
+		if (ctl->granularity==PAPI_GRN_SYS) {
+			pid = -1;
+		}
+		else {
+			pid = 0;
+		}
 	}
 
 	for( i = 0; i < ctl->num_events; i++ ) {
@@ -1083,14 +1102,18 @@ static int
 _pe_rdpmc_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 		long long **events, int flags )
 {
+	long long papi_pe_buffer[READ_BUFFER_SIZE];
+
 	SUBDBG("ENTER: ctx: %p, ctl: %p, events: %p, flags: %#x\n",
 		ctx, ctl, events, flags);
 
 	( void ) flags;			/*unused */
 	( void ) ctx;			/*unused */
+	( void ) papi_pe_buffer;	/*unused */
 	int i;
 	pe_control_t *pe_ctl = ( pe_control_t *) ctl;
-	unsigned long long count, enabled, running, adjusted;
+	unsigned long long count, enabled = 0, running = 0, adjusted;
+	int errors=0;
 
 	/* we must read each counter individually */
 	for ( i = 0; i < pe_ctl->num_events; i++ ) {
@@ -1098,14 +1121,25 @@ _pe_rdpmc_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 		count = mmap_read_self(pe_ctl->events[i].mmap_buf,
 						&enabled,&running);
 
-		/* TODO: error checking? */
+		if (count==0xffffffffffffffffULL) {
+			errors++;
+		}
 
 		/* Handle multiplexing case */
-		if (enabled!=running) {
+		if (enabled == running) {
+			/* no adjustment needed */
+		}
+		else if (enabled && running) {
 			adjusted = (enabled * 128LL) / running;
 			adjusted = adjusted * count;
 			adjusted = adjusted / 128LL;
 			count = adjusted;
+		} else {
+			/* This should not happen, but we have had it reported */
+			SUBDBG("perf_event kernel bug(?) count, enabled, "
+				"running: %lld, %lld, %lld\n",
+				papi_pe_buffer[0],enabled,running);
+
 		}
 
 		pe_ctl->counts[i] = count;
@@ -1115,6 +1149,8 @@ _pe_rdpmc_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 
 	SUBDBG("EXIT: *events: %p\n", *events);
 
+	if (errors) return PAPI_ESYS;
+
 	return PAPI_OK;
 }
 
@@ -1135,7 +1171,7 @@ _pe_read_multiplexed( pe_control_t *pe_ctl )
 				papi_pe_buffer,
 				sizeof ( papi_pe_buffer ) );
 		if ( ret == -1 ) {
-			PAPIERROR("read returned an error: ",
+			PAPIERROR("read returned an error: %s",
 					strerror( errno ));
 			return PAPI_ESYS;
 		}
@@ -1203,7 +1239,7 @@ _pe_read_nogroup( pe_control_t *pe_ctl ) {
 				papi_pe_buffer,
 				sizeof ( papi_pe_buffer ) );
 		if ( ret == -1 ) {
-			PAPIERROR("read returned an error: ",
+			PAPIERROR("read returned an error: %s",
 				strerror( errno ));
 			return PAPI_ESYS;
 		}
@@ -1241,10 +1277,16 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 	int i, j, ret = -1;
 	pe_control_t *pe_ctl = ( pe_control_t *) ctl;
 	long long papi_pe_buffer[READ_BUFFER_SIZE];
+	int result;
 
 	/* Handle fast case */
+	/* FIXME: we fallback to slow reads if *any* event in eventset fails */
+	/*        in theory we could only fall back for the one event        */
+	/*        but that makes the code more complicated.                  */
 	if ((_perf_event_vector.cmp_info.fast_counter_read) && (!pe_ctl->inherit)) {
-		return _pe_rdpmc_read( ctx, ctl, events, flags);
+		result=_pe_rdpmc_read( ctx, ctl, events, flags);
+		/* if successful we are done, otherwise fall back to read */
+		if (result==PAPI_OK) return PAPI_OK;
 	}
 
 	/* Handle case where we are multiplexing */
@@ -1274,7 +1316,7 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 			sizeof ( papi_pe_buffer ) );
 
 		if ( ret == -1 ) {
-			PAPIERROR("read returned an error: ",
+			PAPIERROR("read returned an error: %s",
 				strerror( errno ));
 			return PAPI_ESYS;
 		}
@@ -1627,6 +1669,7 @@ _pe_ctl( hwd_context_t *ctx, int code, _papi_int_option_t *option )
 	      return ret;
 	   }
 
+	   pe_ctl->attached = 1;
 	   pe_ctl->tid = option->attach.tid;
 
 	   /* If events have been already been added, something may */
@@ -1639,7 +1682,9 @@ _pe_ctl( hwd_context_t *ctx, int code, _papi_int_option_t *option )
       case PAPI_DETACH:
 	   pe_ctl = ( pe_control_t *) ( option->attach.ESI->ctl_state );
 
+	   pe_ctl->attached = 0;
 	   pe_ctl->tid = 0;
+
 	   return PAPI_OK;
 
       case PAPI_CPU_ATTACH:
@@ -1653,11 +1698,6 @@ _pe_ctl( hwd_context_t *ctx, int code, _papi_int_option_t *option )
 	   }
 	   /* looks like we are allowed so set cpu number */
 
-	   /* this tells the kernel not to count for a thread   */
-	   /* should we warn if we try to set both?  perf_event */
-	   /* will reject it.                                   */
-	   pe_ctl->tid = -1;
-
 	   pe_ctl->cpu = option->cpu.cpu_num;
 
 	   return PAPI_OK;
@@ -1673,7 +1713,7 @@ _pe_ctl( hwd_context_t *ctx, int code, _papi_int_option_t *option )
 	      return ret;
 	   }
 	   /* looks like we are allowed, so set event set level counting domains */
-       pe_ctl->domain = option->domain.domain;
+	pe_ctl->domain = option->domain.domain;
 	   return PAPI_OK;
 
       case PAPI_GRANUL:
@@ -1898,7 +1938,7 @@ _pe_dispatch_timer( int n, hwd_siginfo_t *info, void *uc)
 	( void ) n;                           /*unused */
 	_papi_hwi_context_t hw_context;
 	int found_evt_idx = -1, fd = info->si_fd;
-	caddr_t address;
+	vptr_t address;
 	ThreadInfo_t *thread = _papi_hwi_lookup_thread( 0 );
 	int i;
 	pe_control_t *ctl;
@@ -2027,7 +2067,7 @@ _pe_dispatch_timer( int n, hwd_siginfo_t *info, void *uc)
 	*/
 
 		_papi_hwi_dispatch_overflow_signal( ( void * ) &hw_context,
-					( caddr_t ) ( unsigned long ) ip,
+					( vptr_t ) ( unsigned long ) ip,
 					NULL, ( 1 << found_evt_idx ), 0,
 					&thread, cidx );
 
@@ -2035,7 +2075,7 @@ _pe_dispatch_timer( int n, hwd_siginfo_t *info, void *uc)
 
 	/* Restart the counters */
 	if (ioctl( fd, PERF_EVENT_IOC_REFRESH, PAPI_REFRESH_VALUE ) == -1) {
-		PAPIERROR( "overflow refresh failed", 0 );
+		PAPIERROR("overflow refresh failed");
 	}
 }
 
@@ -2338,6 +2378,7 @@ _pe_handle_paranoid(papi_vector_t *component) {
 	FILE *fff;
 	int paranoid_level;
 	int retval;
+   char *strCpy;
 
 	/* The is the official way to detect if perf_event support exists */
 	/* The file is called perf_counter_paranoid on 2.6.31             */
@@ -2345,9 +2386,10 @@ _pe_handle_paranoid(papi_vector_t *component) {
 
 	fff=fopen("/proc/sys/kernel/perf_event_paranoid","r");
 	if (fff==NULL) {
-		strncpy(component->cmp_info.disabled_reason,
+		strCpy=strncpy(component->cmp_info.disabled_reason,
 			"perf_event support not detected",PAPI_MAX_STR_LEN);
-		return PAPI_ENOCMP;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		return PAPI_ECMP;
 	}
 
 	/* 3 (vendor patch) means completely disabled */
@@ -2360,9 +2402,10 @@ _pe_handle_paranoid(papi_vector_t *component) {
 	fclose(fff);
 
 	if (paranoid_level==3) {
-		strncpy(component->cmp_info.disabled_reason,
+		strCpy=strncpy(component->cmp_info.disabled_reason,
 			"perf_event support disabled by Linux with paranoid=3",PAPI_MAX_STR_LEN);
-		return PAPI_ENOCMP;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		return PAPI_ECMP;
 	}
 
 	if ((paranoid_level==2) && (getuid()!=0)) {
@@ -2424,12 +2467,14 @@ _pe_init_component( int cidx )
 {
 
 	int retval;
+   char *strCpy;
 
 	our_cidx=cidx;
 
 	/* Update component behavior based on paranoid setting */
 	retval=_pe_handle_paranoid(_papi_hwd[cidx]);
-	if (retval!=PAPI_OK) return retval;
+   
+	if (retval!=PAPI_OK) goto fn_fail; // disabled_reason handled by _pe_handle_paranoid.
 
 #if (OBSOLETE_WORKAROUNDS==1)
 	/* Handle any kernel version related workarounds */
@@ -2439,9 +2484,10 @@ _pe_init_component( int cidx )
 	/* Setup mmtimers, if appropriate */
 	retval=mmtimer_setup();
 	if (retval) {
-		strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+		strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 			"Error initializing mmtimer",PAPI_MAX_STR_LEN);
-		return retval;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		goto fn_fail;
 	}
 
 	/* Set the overflow signal */
@@ -2470,10 +2516,10 @@ _pe_init_component( int cidx )
 	/* Run the libpfm4-specific setup */
 	retval = _papi_libpfm4_init(_papi_hwd[cidx]);
 	if (retval) {
-
-		strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+		strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 			"Error initializing libpfm4",PAPI_MAX_STR_LEN);
-		return retval;
+      if (strCpy == NULL) HANDLE_STRING_ERROR;
+		goto fn_fail;
 
 	}
 
@@ -2486,38 +2532,43 @@ _pe_init_component( int cidx )
 	if (retval) {
 		switch(retval) {
 			case PAPI_ENOMEM:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 memory allocation",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			case PAPI_ENOSUPP:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 no PMUs found",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
-			case PAPI_ENOCMP:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+			case PAPI_ECMP:
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 no default PMU found",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			case PAPI_ECOUNT:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error libpfm4 too many default PMUs found",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			case PAPI_ENOEVNT:
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Error loading preset events",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 				break;
 			default:
-				printf("PAPI error %d\n",retval);
-				strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+				strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 					"Unknown libpfm4 related error",
 					PAPI_MAX_STR_LEN);
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
 
 		}
-		return retval;
+        goto fn_fail;
 	}
 
 	/* Detect NMI watchdog which can steal counters */
@@ -2533,7 +2584,11 @@ _pe_init_component( int cidx )
 	/* check for exclude_guest issue */
 	check_exclude_guest();
 
-	return PAPI_OK;
+  fn_exit:
+    _papi_hwd[cidx]->cmp_info.disabled = retval;
+    return retval;
+  fn_fail:
+    goto fn_exit;
 
 }
 
