@@ -60,7 +60,13 @@ static int our_cidx;
 #define PERF_EVENTS_OPENED  0x01
 #define PERF_EVENTS_RUNNING 0x02
 
+// The following macro follows if a string function has an error. It should 
+// never happen; but it is necessary to prevent compiler warnings. We print 
+// something just in case there is programmer error in invoking the function.
+#define HANDLE_STRING_ERROR {fprintf(stderr,"%s:%i unexpected string function error.\n",__FILE__,__LINE__); exit(-1);}
+
 static int _peu_set_domain( hwd_control_state_t *ctl, int domain);
+static int _peu_shutdown_component( void );
 
 
 
@@ -591,6 +597,7 @@ _peu_init_component( int cidx )
    int paranoid_level;
 
    FILE *fff;
+   char *strCpy;
 
    our_cidx=cidx;
 
@@ -600,9 +607,11 @@ _peu_init_component( int cidx )
 
    fff=fopen("/proc/sys/kernel/perf_event_paranoid","r");
    if (fff==NULL) {
-     strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+     strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 	    "perf_event support not detected",PAPI_MAX_STR_LEN);
-     return PAPI_ENOCMP;
+     if (strCpy == NULL) HANDLE_STRING_ERROR;
+     retval = PAPI_ECMP;
+     goto fn_fail;
    }
    retval=fscanf(fff,"%d",&paranoid_level);
    if (retval!=1) fprintf(stderr,"Error reading paranoid level\n");
@@ -613,9 +622,12 @@ _peu_init_component( int cidx )
 
    retval = _papi_libpfm4_init(_papi_hwd[cidx]);
    if (retval) {
-     strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+     strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 	     "Error initializing libpfm4",PAPI_MAX_STR_LEN);
-     return PAPI_ENOCMP;
+     _peu_shutdown_component( );
+     if (strCpy == NULL) HANDLE_STRING_ERROR;
+     retval = PAPI_ECMP;
+     goto fn_fail;
    }
 
 
@@ -625,17 +637,23 @@ _peu_init_component( int cidx )
 			       &uncore_native_event_table,
                                PMU_TYPE_UNCORE);
    if (retval) {
-     strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+     strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 	     "Error setting up libpfm4",PAPI_MAX_STR_LEN);
-     return PAPI_ENOCMP;
+     _peu_shutdown_component( );
+     if (strCpy == NULL) HANDLE_STRING_ERROR;
+     retval = PAPI_ECMP;
+     goto fn_fail;
    }
 
    /* Check if no uncore events found */
 
    if (_papi_hwd[cidx]->cmp_info.num_native_events==0) {
-     strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+     strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 	     "No uncore PMUs or events found",PAPI_MAX_STR_LEN);
-     return PAPI_ENOCMP;
+     _peu_shutdown_component( );
+     if (strCpy == NULL) HANDLE_STRING_ERROR;
+     retval = PAPI_ECMP;
+     goto fn_fail;
    }
 
    /* Check if we have enough permissions for uncore */
@@ -646,13 +664,20 @@ _peu_init_component( int cidx )
    /* -1 means no restrictions                 */
 
    if ((paranoid_level>0) && (getuid()!=0)) {
-      strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
+      strCpy=strncpy(_papi_hwd[cidx]->cmp_info.disabled_reason,
 	    "Insufficient permissions for uncore access.  Set /proc/sys/kernel/perf_event_paranoid to 0 or run as root.",
 	    PAPI_MAX_STR_LEN);
-     return PAPI_ENOCMP;
+      _peu_shutdown_component( );
+     if (strCpy == NULL) HANDLE_STRING_ERROR;
+     retval = PAPI_ECMP;
+     goto fn_fail;
    }
 
-   return PAPI_OK;
+  fn_exit:
+   _papi_hwd[cidx]->cmp_info.disabled = retval;
+   return retval;
+  fn_fail:
+   goto fn_exit;
 
 }
 
@@ -907,7 +932,7 @@ _peu_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
          ret = read( pe_ctl->events[i].event_fd, papi_pe_buffer,
 		    sizeof ( papi_pe_buffer ) );
          if ( ret == -1 ) {
-	    PAPIERROR("read returned an error: ", strerror( errno ));
+	    PAPIERROR("read returned an error: %s", strerror( errno ));
        SUBDBG("EXIT: PAPI_ESYS\n");
 	    return PAPI_ESYS;
 	 }
@@ -965,7 +990,7 @@ _peu_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
          ret = read( pe_ctl->events[i].event_fd, papi_pe_buffer, 
 		    sizeof ( papi_pe_buffer ) );
          if ( ret == -1 ) {
-	    PAPIERROR("read returned an error: ", strerror( errno ));
+	    PAPIERROR("read returned an error: %s", strerror( errno ));
        SUBDBG("EXIT: PAPI_ESYS\n");
 	    return PAPI_ESYS;
 	 }
@@ -1002,7 +1027,7 @@ _peu_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 		  sizeof ( papi_pe_buffer ) );
 
       if ( ret == -1 ) {
-	 PAPIERROR("read returned an error: ", strerror( errno ));
+	 PAPIERROR("read returned an error: %s", strerror( errno ));
        SUBDBG("EXIT: PAPI_ESYS\n");
 	 return PAPI_ESYS;
       }
