@@ -29,6 +29,24 @@
 #include "papi_vector.h"
 #include "papi_memory.h"
 
+// The following macro follows if a string function has an error. It should 
+// never happen; but it is necessary to prevent compiler warnings. We print 
+// something just in case there is programmer error in invoking the function.
+#define HANDLE_STRING_ERROR {fprintf(stderr,"%s:%i unexpected string function error.\n",__FILE__,__LINE__); exit(-1);}
+
+
+/***************/
+/* AMD Support */
+/***************/
+#define MSR_AMD_RAPL_POWER_UNIT                 0xc0010299
+
+#define MSR_AMD_PKG_ENERGY_STATUS               0xc001029B
+#define MSR_AMD_PP0_ENERGY_STATUS               0xc001029A
+
+
+/*****************/
+/* Intel support */
+/*****************/
 
 /*
  * Platform specific RAPL Domains.
@@ -36,19 +54,18 @@
  * And DRAM RAPL Domain is supported on 062D only
  */
 
-
 /* RAPL defines */
-#define MSR_RAPL_POWER_UNIT             0x606
+#define MSR_INTEL_RAPL_POWER_UNIT	0x606
 
 /* Package */
 #define MSR_PKG_RAPL_POWER_LIMIT        0x610
-#define MSR_PKG_ENERGY_STATUS           0x611
+#define MSR_INTEL_PKG_ENERGY_STATUS     0x611
 #define MSR_PKG_PERF_STATUS             0x613
 #define MSR_PKG_POWER_INFO              0x614
 
 /* PP0 */
 #define MSR_PP0_POWER_LIMIT             0x638
-#define MSR_PP0_ENERGY_STATUS           0x639
+#define MSR_INTEL_PP0_ENERGY_STATUS     0x639
 #define MSR_PP0_POLICY                  0x63A
 #define MSR_PP0_PERF_STATUS             0x63B
 
@@ -64,7 +81,7 @@
 #define MSR_DRAM_POWER_INFO             0x61C
 
 /* PSYS RAPL Domain */
-#define MSR_PLATFORM_ENERGY_STATUS	0x64d
+#define MSR_PLATFORM_ENERGY_STATUS      0x64d
 
 /* RAPL bitsmasks */
 #define POWER_UNIT_OFFSET          0
@@ -117,10 +134,16 @@ typedef struct _rapl_control_state
   long long lastupdate;
 } _rapl_control_state_t;
 
+// The _ENERGY_ counters should return a monotonically increasing
+// value from the _start point, but the hardware only returns a
+// uint32_t that may wrap. We keep a start_value which is reset at
+// _start and every read, handle overflows of the uint32_t, and
+// accumulate a uint64_t which we return.
 
 typedef struct _rapl_context
 {
   long long start_value[RAPL_MAX_COUNTERS];
+  long long accumulated_value[RAPL_MAX_COUNTERS];
   _rapl_control_state_t state;
 } _rapl_context_t;
 
@@ -139,6 +162,7 @@ static int num_packages=0,num_cpus=0;
 
 int power_divisor,time_divisor;
 int cpu_energy_divisor,dram_energy_divisor;
+unsigned int msr_rapl_power_unit;
 
 #define PACKAGE_ENERGY      	0
 #define PACKAGE_THERMAL     	1
@@ -158,16 +182,17 @@ int cpu_energy_divisor,dram_energy_divisor;
 /***************************************************************************/
 
 
-static long long read_msr(int fd, int which) {
+static long long read_msr(int fd, unsigned int which) {
 
-  uint64_t data;
+	uint64_t data;
 
-  if ( fd<0 || pread(fd, &data, sizeof data, which) != sizeof data ) {
-    perror("rdmsr:pread");
-    exit(127);
-  }
+	if ( fd<0 || pread(fd, &data, sizeof data, which) != sizeof data ) {
+		perror("rdmsr:pread");
+		fprintf(stderr,"rdmsr error, msr %x\n",which);
+		exit(127);
+	}
 
-  return (long long)data;
+	return (long long)data;
 }
 
 static int open_fd(int offset) {
@@ -306,9 +331,12 @@ _rapl_init_thread( hwd_context_t *ctx )
 static int
 _rapl_init_component( int cidx )
 {
+    int retval = PAPI_OK;
      int i,j,k,fd;
      FILE *fff;
      char filename[BUFSIZ];
+     long unsigned int strErr;
+     char *strCpy;
 
 	int package_avail, dram_avail, pp0_avail, pp1_avail, psys_avail;
 	int different_units;
@@ -322,6 +350,9 @@ _rapl_init_component( int cidx )
      int packages[nr_cpus];
      int cpu_to_use[nr_cpus];
 
+	unsigned int msr_pkg_energy_status,msr_pp0_energy_status;
+
+
      /* Fill with sentinel values */
      for (i=0; i<nr_cpus; ++i) {
        packages[i] = -1;
@@ -329,28 +360,46 @@ _rapl_init_component( int cidx )
      }
 
 
-     /* check if Intel processor */
-     hw_info=&(_papi_hwi_system_info.hw_info);
+	/* check if supported processor */
+	hw_info=&(_papi_hwi_system_info.hw_info);
 
-     /* Ugh can't use PAPI_get_hardware_info() if
-	PAPI library not done initializing yet */
+	/* Ugh can't use PAPI_get_hardware_info() if
+		PAPI library not done initializing yet */
 
-     if (hw_info->vendor!=PAPI_VENDOR_INTEL) {
-        strncpy(_rapl_vector.cmp_info.disabled_reason,
-		"Not an Intel processor",PAPI_MAX_STR_LEN);
-        return PAPI_ENOSUPP;
-     }
-
-	/* Make sure it is a family 6 Intel Chip */
-	if (hw_info->cpuid_family!=6) {
-		/* Not a family 6 machine */
-		strncpy(_rapl_vector.cmp_info.disabled_reason,
-			"CPU family not supported",PAPI_MAX_STR_LEN);
-		return PAPI_ENOIMPL;
+	switch(hw_info->vendor) {
+		case PAPI_VENDOR_INTEL:
+		case PAPI_VENDOR_AMD:
+			break;
+		default:
+			strCpy=strncpy(_rapl_vector.cmp_info.disabled_reason,
+			"Not a supported processor",PAPI_MAX_STR_LEN);
+			_rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+         if (strCpy == NULL) HANDLE_STRING_ERROR;
+         retval = PAPI_ENOSUPP;
+         goto fn_fail;
 	}
 
-	/* Detect RAPL support */
-	switch(hw_info->cpuid_model) {
+
+	/* Make sure it is a family 6 Intel Chip */
+
+	if (hw_info->vendor==PAPI_VENDOR_INTEL) {
+
+		msr_rapl_power_unit=MSR_INTEL_RAPL_POWER_UNIT;
+		msr_pkg_energy_status=MSR_INTEL_PKG_ENERGY_STATUS;
+		msr_pp0_energy_status=MSR_INTEL_PP0_ENERGY_STATUS;
+
+		if (hw_info->cpuid_family!=6) {
+			/* Not a family 6 machine */
+			strCpy=strncpy(_rapl_vector.cmp_info.disabled_reason,
+				"CPU family not supported",PAPI_MAX_STR_LEN);
+			_rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+         if (strCpy == NULL) HANDLE_STRING_ERROR;
+         retval = PAPI_ENOIMPL;
+         goto fn_fail;
+		}
+
+		/* Detect RAPL support */
+		switch(hw_info->cpuid_model) {
 
 		/* Desktop / Laptops */
 
@@ -436,28 +485,68 @@ _rapl_init_component( int cidx )
 			break;
 
 		default:	/* not a supported model */
-			strncpy(_rapl_vector.cmp_info.disabled_reason,
+			strCpy=strncpy(_rapl_vector.cmp_info.disabled_reason,
 				"CPU model not supported",
 				PAPI_MAX_STR_LEN);
-			return PAPI_ENOIMPL;
+			_rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+         if (strCpy == NULL) HANDLE_STRING_ERROR;
+         retval = PAPI_ENOIMPL;
+         goto fn_fail;
+		}
 	}
 
+	if (hw_info->vendor==PAPI_VENDOR_AMD) {
+
+		msr_rapl_power_unit=MSR_AMD_RAPL_POWER_UNIT;
+		msr_pkg_energy_status=MSR_AMD_PKG_ENERGY_STATUS;
+		msr_pp0_energy_status=MSR_AMD_PP0_ENERGY_STATUS;
+
+		if (hw_info->cpuid_family!=0x17) {
+			/* Not a family 17h machine */
+			strCpy=strncpy(_rapl_vector.cmp_info.disabled_reason,
+				"CPU family not supported",PAPI_MAX_STR_LEN);
+			_rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+         if (strCpy == NULL) HANDLE_STRING_ERROR;
+         retval = PAPI_ENOIMPL;
+         goto fn_fail;
+		}
+
+		package_avail=1;
+		pp0_avail=1;		/* Doesn't work on EPYC? */
+		pp1_avail=0;
+		dram_avail=0;
+		psys_avail=0;
+		different_units=0;
+	}
+
+
      /* Detect how many packages */
+     // Some code below may be flagged by Coverity due to uninitialized array
+     // entries of cpu_to_use[]. This is not a bug; the 'filename' listed below
+     // will have 'cpu0', 'cpu1', sequentially on up to the maximum.  Coverity
+     // cannot know that, so its code analysis allows the possibility that the
+     // cpu_to_use[] array is only partially filled in. [Tony C. 11-27-19].
+
      j=0;
      while(1) {
        int num_read;
 
-       sprintf(filename,
+       strErr=snprintf(filename, BUFSIZ, 
 	       "/sys/devices/system/cpu/cpu%d/topology/physical_package_id",j);
+       filename[BUFSIZ-1]=0;
+       if (strErr > BUFSIZ) HANDLE_STRING_ERROR;
        fff=fopen(filename,"r");
        if (fff==NULL) break;
        num_read=fscanf(fff,"%d",&package);
        fclose(fff);
        if (num_read!=1) {
-    		 strcpy(_rapl_vector.cmp_info.disabled_reason, "Error reading file: ");
-    		 strncat(_rapl_vector.cmp_info.disabled_reason, filename, PAPI_MAX_STR_LEN - strlen(_rapl_vector.cmp_info.disabled_reason) - 1);
+    		 strCpy=strcpy(_rapl_vector.cmp_info.disabled_reason, "Error reading file: ");
+          if (strCpy == NULL) HANDLE_STRING_ERROR;
+    		 strCpy=strncat(_rapl_vector.cmp_info.disabled_reason, filename, PAPI_MAX_STR_LEN - strlen(_rapl_vector.cmp_info.disabled_reason) - 1);
     		 _rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1] = '\0';
-    		 return PAPI_ESYS;
+          if (strCpy == NULL) HANDLE_STRING_ERROR;
+          retval = PAPI_ESYS;
+          goto fn_fail;
        }
 
        /* Check if a new package */
@@ -470,9 +559,12 @@ _rapl_init_component( int cidx )
          }
        } else {
 	 SUBDBG("Package outside of allowed range\n");
-	 strncpy(_rapl_vector.cmp_info.disabled_reason,
+	 strCpy=strncpy(_rapl_vector.cmp_info.disabled_reason,
 		"Package outside of allowed range",PAPI_MAX_STR_LEN);
-	 return PAPI_ESYS;
+	 _rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+    if (strCpy == NULL) HANDLE_STRING_ERROR;
+    retval = PAPI_ESYS;
+    goto fn_fail;
        }
 
        j++;
@@ -481,9 +573,12 @@ _rapl_init_component( int cidx )
 
      if (num_packages==0) {
         SUBDBG("Can't access /dev/cpu/*/<msr_safe | msr>\n");
-	strncpy(_rapl_vector.cmp_info.disabled_reason,
+    strCpy=strncpy(_rapl_vector.cmp_info.disabled_reason,
 		"Can't access /dev/cpu/*/<msr_safe | msr>",PAPI_MAX_STR_LEN);
-	return PAPI_ESYS;
+    _rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+    if (strCpy == NULL) HANDLE_STRING_ERROR;
+    retval = PAPI_ESYS;
+    goto fn_fail;
      }
 
      SUBDBG("Found %d packages with %d cpus\n",num_packages,num_cpus);
@@ -491,24 +586,33 @@ _rapl_init_component( int cidx )
      /* Init fd_array */
 
      fd_array=papi_calloc(num_cpus, sizeof(struct fd_array_t));
-     if (fd_array==NULL) return PAPI_ENOMEM;
+     if (fd_array==NULL) {
+         retval = PAPI_ENOMEM;
+         goto fn_fail;
+     }
 
      fd=open_fd(cpu_to_use[0]);
      if (fd<0) {
-        sprintf(_rapl_vector.cmp_info.disabled_reason,
-		"Can't open fd for cpu0: %s",strerror(errno));
-        return PAPI_ESYS;
+        strErr=snprintf(_rapl_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN,
+        "Can't open fd for cpu0: %s",strerror(errno));
+        _rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+        retval = PAPI_ESYS;
+        goto fn_fail;
      }
 
      /* Verify needed MSR is readable. In a guest VM it may not be readable*/
-     if (pread(fd, &result, sizeof result, MSR_RAPL_POWER_UNIT) != sizeof result ) {
-        strncpy(_rapl_vector.cmp_info.disabled_reason,
+     if (pread(fd, &result, sizeof result, msr_rapl_power_unit) != sizeof result ) {
+        strCpy=strncpy(_rapl_vector.cmp_info.disabled_reason,
                "Unable to access RAPL registers",PAPI_MAX_STR_LEN);
-        return PAPI_ESYS;
+        _rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+        retval = PAPI_ESYS;
+        goto fn_fail;
      }
 
      /* Calculate the units used */
-     result=read_msr(fd,MSR_RAPL_POWER_UNIT);
+     result=read_msr(fd,msr_rapl_power_unit);
 
      /* units are 0.5^UNIT_VALUE */
      /* which is the same as 1/(2^UNIT_VALUE) */
@@ -540,34 +644,58 @@ _rapl_init_component( int cidx )
                  (pp0_avail*num_packages) +
                  (pp1_avail*num_packages) +
                  (dram_avail*num_packages) +
-		(psys_avail*num_packages) +
-                 (4*num_packages)) * 2;
+		(psys_avail*num_packages)) * 2;
+
+	if (hw_info->vendor==PAPI_VENDOR_INTEL) {
+		num_events+=(4*num_packages) * 2;
+	}
 
      rapl_native_events = (_rapl_native_event_entry_t*)
           papi_calloc(num_events, sizeof(_rapl_native_event_entry_t));
-
+     if (rapl_native_events == NULL) {
+        strErr=snprintf(_rapl_vector.cmp_info.disabled_reason, PAPI_MAX_STR_LEN,
+        "%s:%i rapl_native_events papi_calloc for %lu bytes failed.", __FILE__, __LINE__, num_events*sizeof(_rapl_native_event_entry_t));
+        _rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+        retval = PAPI_ENOMEM;
+        goto fn_fail;
+     }
 
      i = 0;
      k = num_events/2;
 
      /* Create events for package power info */
 
+	if (hw_info->vendor==PAPI_VENDOR_INTEL)
      for(j=0;j<num_packages;j++) {
-     	sprintf(rapl_native_events[i].name,
+        strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN, 
 			"THERMAL_SPEC_CNT:PACKAGE%d",j);
-		sprintf(rapl_native_events[i].description,
+        rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+
+        strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   "Thermal specification in counts; package %d",j);
+        rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[i].fd_offset=cpu_to_use[j];
 		rapl_native_events[i].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[i].resources.selector = i + 1;
 		rapl_native_events[i].type=PACKAGE_THERMAL_CNT;
 		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-     	sprintf(rapl_native_events[k].name,
+        strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 			"THERMAL_SPEC:PACKAGE%d",j);
-		strncpy(rapl_native_events[k].units,"W",PAPI_MIN_STR_LEN);
-		sprintf(rapl_native_events[k].description,
+        rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+
+        strCpy=strncpy(rapl_native_events[k].units,"W",PAPI_MIN_STR_LEN);
+        rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+
+        strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   "Thermal specification for package %d",j);
+        rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[k].fd_offset=cpu_to_use[j];
 		rapl_native_events[k].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[k].resources.selector = k + 1;
@@ -578,22 +706,33 @@ _rapl_init_component( int cidx )
 		k++;
      }
 
+	if (hw_info->vendor==PAPI_VENDOR_INTEL)
      for(j=0;j<num_packages;j++) {
-		sprintf(rapl_native_events[i].name,
+        strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 			"MINIMUM_POWER_CNT:PACKAGE%d",j);
-		sprintf(rapl_native_events[i].description,
+        rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+        strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   "Minimum power in counts; package %d",j);
+        rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[i].fd_offset=cpu_to_use[j];
 		rapl_native_events[i].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[i].resources.selector = i + 1;
 		rapl_native_events[i].type=PACKAGE_MINIMUM_CNT;
 		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-		sprintf(rapl_native_events[k].name,
+        strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 			"MINIMUM_POWER:PACKAGE%d",j);
-		strncpy(rapl_native_events[k].units,"W",PAPI_MIN_STR_LEN);
-		sprintf(rapl_native_events[k].description,
+        rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+        strCpy=strncpy(rapl_native_events[k].units,"W",PAPI_MIN_STR_LEN);
+        rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+        strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   "Minimum power for package %d",j);
+        rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[k].fd_offset=cpu_to_use[j];
 		rapl_native_events[k].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[k].resources.selector = k + 1;
@@ -604,22 +743,33 @@ _rapl_init_component( int cidx )
 		k++;
      }
 
+	if (hw_info->vendor==PAPI_VENDOR_INTEL)
      for(j=0;j<num_packages;j++) {
-		sprintf(rapl_native_events[i].name,
+        strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 			"MAXIMUM_POWER_CNT:PACKAGE%d",j);
-		sprintf(rapl_native_events[i].description,
+        rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+        strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   "Maximum power in counts; package %d",j);
+        rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[i].fd_offset=cpu_to_use[j];
 		rapl_native_events[i].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[i].resources.selector = i + 1;
 		rapl_native_events[i].type=PACKAGE_MAXIMUM_CNT;
 		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-		sprintf(rapl_native_events[k].name,
+        strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 			"MAXIMUM_POWER:PACKAGE%d",j);
-		strncpy(rapl_native_events[k].units,"W",PAPI_MIN_STR_LEN);
-		sprintf(rapl_native_events[k].description,
+        rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+        strCpy=strncpy(rapl_native_events[k].units,"W",PAPI_MIN_STR_LEN);
+        rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+        if (strCpy == NULL) HANDLE_STRING_ERROR;
+        strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   "Maximum power for package %d",j);
+        rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+        if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[k].fd_offset=cpu_to_use[j];
 		rapl_native_events[k].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[k].resources.selector = k + 1;
@@ -630,22 +780,33 @@ _rapl_init_component( int cidx )
 		k++;
      }
 
+	if (hw_info->vendor==PAPI_VENDOR_INTEL)
      for(j=0;j<num_packages;j++) {
-		sprintf(rapl_native_events[i].name,
+         strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 			"MAXIMUM_TIME_WINDOW_CNT:PACKAGE%d",j);
-		sprintf(rapl_native_events[i].description,
+         rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+         if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+         strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   "Maximum time window in counts; package %d",j);
+         rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+         if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[i].fd_offset=cpu_to_use[j];
 		rapl_native_events[i].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[i].resources.selector = i + 1;
 		rapl_native_events[i].type=PACKAGE_TIME_WINDOW_CNT;
 		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-		sprintf(rapl_native_events[k].name,
+         strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 			"MAXIMUM_TIME_WINDOW:PACKAGE%d",j);
-		strncpy(rapl_native_events[k].units,"s",PAPI_MIN_STR_LEN);
-		sprintf(rapl_native_events[k].description,
+         rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+         if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+         strCpy=strncpy(rapl_native_events[k].units,"s",PAPI_MIN_STR_LEN);
+         rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+         if (strCpy == NULL) HANDLE_STRING_ERROR;
+         strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   "Maximum time window for package %d",j);
+         rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+         if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 		rapl_native_events[k].fd_offset=cpu_to_use[j];
 		rapl_native_events[k].msr=MSR_PKG_POWER_INFO;
 		rapl_native_events[k].resources.selector = k + 1;
@@ -660,23 +821,33 @@ _rapl_init_component( int cidx )
 
      if (package_avail) {
         for(j=0;j<num_packages;j++) {
-	   		sprintf(rapl_native_events[i].name,
+            strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 		   		"PACKAGE_ENERGY_CNT:PACKAGE%d",j);
-	   		sprintf(rapl_native_events[i].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   		"Energy used in counts by chip package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[i].fd_offset=cpu_to_use[j];
-	   		rapl_native_events[i].msr=MSR_PKG_ENERGY_STATUS;
+	   		rapl_native_events[i].msr=msr_pkg_energy_status;
 	   		rapl_native_events[i].resources.selector = i + 1;
 	   		rapl_native_events[i].type=PACKAGE_ENERGY_CNT;
 	   		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-	   		sprintf(rapl_native_events[k].name,
+            strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 		   		"PACKAGE_ENERGY:PACKAGE%d",j);
-	   		strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
-	   		sprintf(rapl_native_events[k].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strCpy=strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
+            rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   		"Energy used by chip package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[k].fd_offset=cpu_to_use[j];
-	   		rapl_native_events[k].msr=MSR_PKG_ENERGY_STATUS;
+	   		rapl_native_events[k].msr=msr_pkg_energy_status;
 	   		rapl_native_events[k].resources.selector = k + 1;
 	   		rapl_native_events[k].type=PACKAGE_ENERGY;
 	   		rapl_native_events[k].return_type=PAPI_DATATYPE_UINT64;
@@ -688,21 +859,31 @@ _rapl_init_component( int cidx )
 
      if (pp1_avail) {
         for(j=0;j<num_packages;j++) {
-	   		sprintf(rapl_native_events[i].name,
+            strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 		   		"PP1_ENERGY_CNT:PACKAGE%d",j);
-	   		sprintf(rapl_native_events[i].description,
-		   		"Energy used in counts by Power Plane 1 (Often GPU) on package %d",j);
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
+		   	"Energy used in counts by Power Plane 1 (Often GPU) on package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
            	rapl_native_events[i].fd_offset=cpu_to_use[j];
 	   		rapl_native_events[i].msr=MSR_PP1_ENERGY_STATUS;
 	   		rapl_native_events[i].resources.selector = i + 1;
 	   		rapl_native_events[i].type=PACKAGE_ENERGY_CNT;
 	   		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-	   		sprintf(rapl_native_events[k].name,
+            strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 		   		"PP1_ENERGY:PACKAGE%d",j);
-	   		strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
-	   		sprintf(rapl_native_events[k].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strCpy=strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
+            rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   		"Energy used by Power Plane 1 (Often GPU) on package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
            	rapl_native_events[k].fd_offset=cpu_to_use[j];
 	   		rapl_native_events[k].msr=MSR_PP1_ENERGY_STATUS;
 	   		rapl_native_events[k].resources.selector = k + 1;
@@ -716,21 +897,31 @@ _rapl_init_component( int cidx )
 
      if (dram_avail) {
         for(j=0;j<num_packages;j++) {
-	   		sprintf(rapl_native_events[i].name,
+            strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 		   		"DRAM_ENERGY_CNT:PACKAGE%d",j);
-           	sprintf(rapl_native_events[i].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   		"Energy used in counts by DRAM on package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[i].fd_offset=cpu_to_use[j];
 	   		rapl_native_events[i].msr=MSR_DRAM_ENERGY_STATUS;
 	   		rapl_native_events[i].resources.selector = i + 1;
 	   		rapl_native_events[i].type=PACKAGE_ENERGY_CNT;
 	   		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-	   		sprintf(rapl_native_events[k].name,
+            strErr=snprintf(rapl_native_events[k].name,PAPI_MAX_STR_LEN,
 		   		"DRAM_ENERGY:PACKAGE%d",j);
-	   		strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
-           	sprintf(rapl_native_events[k].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strCpy=strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
+            rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   		"Energy used by DRAM on package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[k].fd_offset=cpu_to_use[j];
 	   		rapl_native_events[k].msr=MSR_DRAM_ENERGY_STATUS;
 	   		rapl_native_events[k].resources.selector = k + 1;
@@ -745,21 +936,31 @@ _rapl_init_component( int cidx )
      if (psys_avail) {
         for(j=0;j<num_packages;j++) {
 
-	   		sprintf(rapl_native_events[i].name,
+         strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 		   		"PSYS_ENERGY_CNT:PACKAGE%d",j);
-           	sprintf(rapl_native_events[i].description,
+         rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+         if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   		"Energy used in counts by SoC on package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[i].fd_offset=cpu_to_use[j];
 	   		rapl_native_events[i].msr=MSR_PLATFORM_ENERGY_STATUS;
 	   		rapl_native_events[i].resources.selector = i + 1;
 	   		rapl_native_events[i].type=PACKAGE_ENERGY_CNT;
 	   		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-	   		sprintf(rapl_native_events[k].name,
+            strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 		   		"PSYS_ENERGY:PACKAGE%d",j);
-	   		strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
-           	sprintf(rapl_native_events[k].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strCpy=strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
+            rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   		"Energy used by SoC on package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[k].fd_offset=cpu_to_use[j];
 	   		rapl_native_events[k].msr=MSR_PLATFORM_ENERGY_STATUS;
 	   		rapl_native_events[k].resources.selector = k + 1;
@@ -773,23 +974,33 @@ _rapl_init_component( int cidx )
 
      if (pp0_avail) {
         for(j=0;j<num_packages;j++) {
-			sprintf(rapl_native_events[i].name,
+            strErr=snprintf(rapl_native_events[i].name, PAPI_MAX_STR_LEN,
 		   		"PP0_ENERGY_CNT:PACKAGE%d",j);
-	   		sprintf(rapl_native_events[i].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[i].description, PAPI_MAX_STR_LEN,
 		   		"Energy used in counts by all cores in package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[i].fd_offset=cpu_to_use[j];
-	   		rapl_native_events[i].msr=MSR_PP0_ENERGY_STATUS;
+	   		rapl_native_events[i].msr=msr_pp0_energy_status;
 	   		rapl_native_events[i].resources.selector = i + 1;
 	   		rapl_native_events[i].type=PACKAGE_ENERGY_CNT;
 	   		rapl_native_events[i].return_type=PAPI_DATATYPE_UINT64;
 
-			sprintf(rapl_native_events[k].name,
+            strErr=snprintf(rapl_native_events[k].name, PAPI_MAX_STR_LEN,
 		   		"PP0_ENERGY:PACKAGE%d",j);
-	   		strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
-	   		sprintf(rapl_native_events[k].description,
+            rapl_native_events[i].name[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
+            strCpy=strncpy(rapl_native_events[k].units,"nJ",PAPI_MIN_STR_LEN);
+            rapl_native_events[k].units[PAPI_MIN_STR_LEN-1]=0;
+            if (strCpy == NULL) HANDLE_STRING_ERROR;
+            strErr=snprintf(rapl_native_events[k].description, PAPI_MAX_STR_LEN,
 		   		"Energy used by all cores in package %d",j);
+            rapl_native_events[i].description[PAPI_MAX_STR_LEN-1]=0;
+            if (strErr > PAPI_MAX_STR_LEN) HANDLE_STRING_ERROR;
 	   		rapl_native_events[k].fd_offset=cpu_to_use[j];
-	   		rapl_native_events[k].msr=MSR_PP0_ENERGY_STATUS;
+	   		rapl_native_events[k].msr=msr_pp0_energy_status;
 	   		rapl_native_events[k].resources.selector = k + 1;
 	   		rapl_native_events[k].type=PACKAGE_ENERGY;
 	   		rapl_native_events[k].return_type=PAPI_DATATYPE_UINT64;
@@ -809,7 +1020,11 @@ _rapl_init_component( int cidx )
      /* Export the component id */
      _rapl_vector.cmp_info.CmpIdx = cidx;
 
-     return PAPI_OK;
+  fn_exit:
+    _papi_hwd[cidx]->cmp_info.disabled = retval;
+     return retval;
+  fn_fail:
+     goto fn_exit;
 }
 
 
@@ -839,9 +1054,11 @@ _rapl_start( hwd_context_t *ctx, hwd_control_state_t *ctl)
   long long now = PAPI_get_real_usec();
   int i;
 
+  
   for( i = 0; i < RAPL_MAX_COUNTERS; i++ ) {
      if ((control->being_measured[i]) && (control->need_difference[i])) {
-        context->start_value[i]=read_rapl_value(i);
+        context->start_value[i]=(read_rapl_value(i) & 0xFFFFFFFF);
+        context->accumulated_value[i]=0;
      }
   }
 
@@ -853,31 +1070,35 @@ _rapl_start( hwd_context_t *ctx, hwd_control_state_t *ctl)
 static int
 _rapl_stop( hwd_context_t *ctx, hwd_control_state_t *ctl )
 {
+   /* read values */
+   _rapl_context_t* context = (_rapl_context_t*) ctx;
+   _rapl_control_state_t* control = (_rapl_control_state_t*) ctl;
+   long long now = PAPI_get_real_usec();
+   int i;
+   long long temp, newstart;
 
-    /* read values */
-    _rapl_context_t* context = (_rapl_context_t*) ctx;
-    _rapl_control_state_t* control = (_rapl_control_state_t*) ctl;
-    long long now = PAPI_get_real_usec();
-    int i;
-    long long temp;
-
-    for ( i = 0; i < RAPL_MAX_COUNTERS; i++ ) {
-		if (control->being_measured[i]) {
-			temp = read_rapl_value(i);
-			if (context->start_value[i])
-			if (control->need_difference[i]) {
-				/* test for wrap around */
-				if (temp < context->start_value[i] ) {
-	        		SUBDBG("Wraparound!\nstart:\t%#016x\ttemp:\t%#016x",
-	        			(unsigned)context->start_value[i], (unsigned)temp);
-	           		temp += (0x100000000 - context->start_value[i]);
-	           		SUBDBG("\tresult:\t%#016x\n", (unsigned)temp);
-		  		} else {
-					temp -= context->start_value[i];
-		  		}
-			}
-			control->count[i] = convert_rapl_energy( i, temp );
-		}
+   for ( i = 0; i < RAPL_MAX_COUNTERS; i++ ) {
+      if (control->being_measured[i]) {
+         temp = read_rapl_value(i);
+         if (control->need_difference[i]) {
+            temp &= 0xFFFFFFFF;
+            newstart = temp;
+            /* test for wrap around */
+            if (temp < context->start_value[i] ) {
+               SUBDBG("Wraparound!\nstart:\t%#016x\ttemp:\t%#016x",
+                  (unsigned)context->start_value[i], (unsigned)temp);
+               temp += (0x100000000 - context->start_value[i]);
+               SUBDBG("\tresult:\t%#016x\n", (unsigned)temp);
+            } else {
+               temp -= context->start_value[i];
+            }
+            // reset the start value, add to accum, set temp for convert call.
+            context->start_value[i]=newstart;
+            context->accumulated_value[i] += temp;
+            temp = context->accumulated_value[i];
+         }
+         control->count[i] = convert_rapl_energy( i, temp );
+      }
     }
     control->lastupdate = now;
     return PAPI_OK;
