@@ -376,11 +376,23 @@ static void
 print_event_flags(pfm_event_info_t *info)
 {
 	int n = 0;
+	int spec = info->is_speculative;
 
 	if (info->is_precise) {
 		printf("[precise] ");
 		n++;
 	}
+
+	if (info->support_hw_smpl) {
+		printf("[hw_smpl] ");
+		n++;
+	}
+
+	if (spec > PFM_EVENT_INFO_SPEC_NA) {
+		printf("[%s] ", spec == PFM_EVENT_INFO_SPEC_TRUE ? "speculative" : "non-speculative");
+		n++;
+	}
+
 	if (!n)
 		printf("None");
 }
@@ -389,6 +401,7 @@ static void
 print_attr_flags(pfm_event_attr_info_t *info)
 {
 	int n = 0;
+	int spec = info->is_speculative;
 
 	if (info->is_dfl) {
 		printf("[default] ");
@@ -400,6 +413,16 @@ print_attr_flags(pfm_event_attr_info_t *info)
 		n++;
 	}
 
+	if (info->support_hw_smpl) {
+		printf("[hw_smpl] ");
+		n++;
+	}
+
+	if (spec > PFM_EVENT_INFO_SPEC_NA) {
+		printf("[%s] ", spec == PFM_EVENT_INFO_SPEC_TRUE ? "speculative" : "non-speculative");
+		n++;
+	}
+
 	if (!n)
 		printf("None ");
 }
@@ -502,7 +525,8 @@ show_info(char *event, regex_t *preg)
 {
 	pfm_pmu_info_t pinfo;
 	pfm_event_info_t info;
-	int i, j, ret, match = 0, pname;
+	pfm_pmu_t j;
+	int i, ret, match = 0, pname;
 	size_t len, l = 0;
 	char *fullname = NULL;
 
@@ -565,7 +589,7 @@ show_info_sorted(char *event, regex_t *preg)
 {
 	pfm_pmu_info_t pinfo;
 	pfm_event_info_t info;
-	unsigned int j;
+	pfm_pmu_t j;
 	int i, ret, n, match = 0;
 	size_t len, l = 0;
 	char *fullname = NULL;
@@ -781,7 +805,8 @@ main(int argc, char **argv)
 	char default_sep[2] = "\t";
 	char *ostr = NULL;
 	char **args;
-	int i, match;
+	pfm_pmu_t i;
+	int match;
 	regex_t preg;
 	int ret, c;
 
