@@ -81,6 +81,9 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&amd64_fam17h_deprecated_support,
 	&amd64_fam17h_zen1_support,
 	&amd64_fam17h_zen2_support,
+	&amd64_fam19h_zen3_support,
+	&amd64_fam19h_zen3_l3_support,
+	&amd64_rapl_support,
 	&intel_core_support,
 	&intel_atom_support,
 	&intel_nhm_support,
@@ -109,6 +112,8 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&intel_skx_support,
 	&intel_clx_support,
 	&intel_icl_support,
+	&intel_icx_support,
+	&intel_spr_support,
 	&intel_rapl_support,
 	&intel_snbep_unc_cb0_support,
 	&intel_snbep_unc_cb1_support,
@@ -458,6 +463,7 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&power7_support,
 	&power8_support,
 	&power9_support,
+	&power10_support,
 	&torrent_support,
     &powerpc_nest_mcs_read_support,
     &powerpc_nest_mcs_write_support,
@@ -494,6 +500,65 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&arm_thunderx2_llc1_support,
 	&arm_thunderx2_ccpi0_support,
 	&arm_thunderx2_ccpi1_support,
+	&arm_n1_support,
+	&arm_n2_support,
+	&arm_hisilicon_kunpeng_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl1_hha2_support,
+	&arm_hisilicon_kunpeng_sccl1_hha3_support,
+	&arm_hisilicon_kunpeng_sccl3_hha0_support,
+	&arm_hisilicon_kunpeng_sccl3_hha1_support,
+	&arm_hisilicon_kunpeng_sccl5_hha6_support,
+	&arm_hisilicon_kunpeng_sccl5_hha7_support,
+	&arm_hisilicon_kunpeng_sccl7_hha4_support,
+	&arm_hisilicon_kunpeng_sccl7_hha5_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c10_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c11_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c12_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c13_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c14_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c15_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c8_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c9_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c0_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c1_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c2_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c3_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c4_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c5_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c6_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c7_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c24_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c25_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c26_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c27_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c28_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c29_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c30_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c31_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c16_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c17_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c18_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c19_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c20_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c21_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c22_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c23_support,
 #endif
 #ifdef CONFIG_PFMLIB_ARCH_ARM64
 	&arm_cortex_a57_support,
@@ -506,6 +571,66 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&arm_thunderx2_llc1_support,
 	&arm_thunderx2_ccpi0_support,
 	&arm_thunderx2_ccpi1_support,
+	&arm_fujitsu_a64fx_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl1_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl3_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl5_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc0_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc1_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc2_support,
+	&arm_hisilicon_kunpeng_sccl7_ddrc3_support,
+	&arm_hisilicon_kunpeng_sccl1_hha2_support,
+	&arm_hisilicon_kunpeng_sccl1_hha3_support,
+	&arm_hisilicon_kunpeng_sccl3_hha0_support,
+	&arm_hisilicon_kunpeng_sccl3_hha1_support,
+	&arm_hisilicon_kunpeng_sccl5_hha6_support,
+	&arm_hisilicon_kunpeng_sccl5_hha7_support,
+	&arm_hisilicon_kunpeng_sccl7_hha4_support,
+	&arm_hisilicon_kunpeng_sccl7_hha5_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c10_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c11_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c12_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c13_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c14_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c15_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c8_support,
+	&arm_hisilicon_kunpeng_sccl1_l3c9_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c0_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c1_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c2_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c3_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c4_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c5_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c6_support,
+	&arm_hisilicon_kunpeng_sccl3_l3c7_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c24_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c25_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c26_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c27_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c28_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c29_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c30_support,
+	&arm_hisilicon_kunpeng_sccl5_l3c31_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c16_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c17_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c18_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c19_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c20_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c21_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c22_support,
+	&arm_hisilicon_kunpeng_sccl7_l3c23_support,
+	&arm_n1_support,
+	&arm_n2_support,
+	&arm_hisilicon_kunpeng_support,
 #endif
 
 #ifdef CONFIG_PFMLIB_ARCH_S390X
@@ -541,6 +666,30 @@ static pfmlib_os_t *pfmlib_oses[]={
  */
 static pfmlib_pmu_t *pfmlib_pmus_map[PFM_PMU_MAX];
 
+/*
+ * A drop-in replacement for strsep(). strsep() is not part of the POSIX
+ * standard, and it is not available on all platforms - in particular it is not
+ * provided by Microsoft's C runtime or by MinGW.
+ */
+static char* pfmlib_strsep(char **stringp, const char *delim)
+{
+	char* token = *stringp;
+	char* end = *stringp;
+
+	if (!end)
+		return NULL;
+
+	while (*end && !strchr(delim, *end))
+		end++;
+
+	if (*end) {
+		*end = '\0';
+		*stringp = end + 1;
+	} else {
+		*stringp = NULL;
+	}
+	return token;
+}
 
 #define pfmlib_for_each_pmu_event(p, e) \
 	for(e=(p)->get_event_first((p)); e != -1; e = (p)->get_event_next((p), e))
@@ -1170,7 +1319,7 @@ pfmlib_parse_event_attr(char *str, pfmlib_event_desc_t *d)
 
 	while(s) {
 	        p = s;
-	        strsep(&p, PFMLIB_ATTR_DELIM);
+	        pfmlib_strsep(&p, PFMLIB_ATTR_DELIM);
 		/* if (p)
 		 *p++ = '\0'; */
 
@@ -1418,7 +1567,7 @@ pfmlib_parse_equiv_event(const char *event, pfmlib_event_desc_t *d)
 		return PFM_ERR_NOMEM;
 
 	p = s;
-	strsep(&p, PFMLIB_ATTR_DELIM);
+	pfmlib_strsep(&p, PFMLIB_ATTR_DELIM);
 	/* if (p)
 	 *p++ = '\0'; */
 
@@ -1490,7 +1639,7 @@ pfmlib_parse_event(const char *event, pfmlib_event_desc_t *d)
 		s = p + strlen(PFMLIB_PMU_DELIM);
 	}
 	p = s;
-	strsep(&p, PFMLIB_ATTR_DELIM);
+	pfmlib_strsep(&p, PFMLIB_ATTR_DELIM);
 	/* if (p)
 	 *p++ = '\0'; */
 
