@@ -692,7 +692,7 @@ user_signal_handler( int hEvtSet, uint64_t address, uint64_t ovfVector, const uc
 	int isHardware = 1;
 	int cidx = _bgq_vectors.cmp_info.CmpIdx;
 	long_long overflow_bit = 0;
-	caddr_t address1;
+	vptr_t address1;
 	_papi_hwi_context_t ctx;
 	ctx.ucontext = ( hwd_ucontext_t * ) pContext;
 	ThreadInfo_t *thread = _papi_hwi_lookup_thread( 0 );
@@ -1057,7 +1057,7 @@ _bgq_init_component( int cidx )
  * Event Name to Native Code
  */
 int
-_bgq_ntv_name_to_code( char *name, unsigned int *event_code )
+_bgq_ntv_name_to_code( const char *name, unsigned int *event_code )
 {
 #ifdef DEBUG_BGQ
 	printf( "_bgq_ntv_name_to_code\n" );
