@@ -1,12 +1,38 @@
-/* $Id: cpumask.h,v 1.11 2004/07/12 22:44:14 mikpe Exp $
+/* $Id: cpumask.h,v 1.6.2.3 2009/01/23 17:01:02 mikpe Exp $
  * Performance-monitoring counters driver.
  * Partial simulation of cpumask_t on non-cpumask_t kernels.
  * Extension to allow inspecting a cpumask_t as array of ulong.
  * Appropriate definition of perfctr_cpus_forbidden_mask.
  *
- * Copyright (C) 2003-2004  Mikael Pettersson
+ * Copyright (C) 2003-2004, 2009  Mikael Pettersson
  */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,1)
+
+/* 2.6.1-rc1 introduced cpus_addr() */
+#ifdef CPU_ARRAY_SIZE
+#define cpus_addr(map)		((map).mask)
+#else
+#define cpus_addr(map)		(&(map))
+#endif
+
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,8) && !defined(cpus_andnot)
+#define cpus_andnot(dst, src1, src2) \
+do { \
+    cpumask_t _tmp2; \
+    _tmp2 = (src2); \
+    cpus_complement(_tmp2); \
+    cpus_and((dst), (src1), _tmp2); \
+} while(0)
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,8) && !defined(CONFIG_SMP)
+#undef cpu_online_map
+#define cpu_online_map	cpumask_of_cpu(0)
+#endif
+
 #ifdef CPU_ARRAY_SIZE
 #define PERFCTR_CPUMASK_NRLONGS	CPU_ARRAY_SIZE
 #else
