@@ -428,7 +428,7 @@ static void print_spaces(int count) {
 /* TESTS_QUIET #define printf hack		*/
 /* FIXME! Revert to printf once we are done converting */
 
-void
+void PAPI_NORETURN
 test_pass( const char *filename )
 {
 	(void)filename;
@@ -463,8 +463,31 @@ test_pass( const char *filename )
 
 }
 
+void PAPI_NORETURN
+test_hl_pass( const char *filename )
+{
+	(void)filename;
+
+	if ( TEST_WARN ) {
+		print_spaces(59);
+		if (TESTS_COLOR) fprintf( stdout, "%s", YELLOW);
+		fprintf( stdout, "PASSED with WARNING");
+		if (TESTS_COLOR) fprintf( stdout, "%s", NORMAL);
+		fprintf( stdout, "\n");
+	}
+	else {
+		if (TESTS_COLOR) fprintf( stdout, "%s",GREEN);
+		fprintf( stdout, "PASSED");
+		if (TESTS_COLOR) fprintf( stdout, "%s",NORMAL);
+		fprintf( stdout, "\n");
+	}
+
+	exit( 0 );
+
+}
+
 /* Use a positive value of retval to simply print an error message */
-void
+void PAPI_NORETURN
 test_fail( const char *file, int line, const char *call, int retval )
 {
 //	int line_pad;
@@ -500,7 +523,9 @@ test_fail( const char *file, int line, const char *call, int retval )
 		fprintf( stdout, "Error in %s: %s\n", call, PAPI_strerror( retval ) );
 	}
 
-//	fprintf( stdout, "\n" );
+   fprintf(stdout, "Some tests require special hardware, permissions, OS, compilers\n"
+                   "or library versions. PAPI may still function perfectly on your \n"
+                   "system without the particular feature being tested here.       \n");
 
 	/* NOTE: Because test_fail is called from thread functions,
 	   calling PAPI_shutdown here could prevent some threads
@@ -555,7 +580,7 @@ test_warn( const char *file, int line, const char *call, int retval )
 	TEST_WARN++;
 }
 
-void
+void PAPI_NORETURN
 test_skip( const char *file, int line, const char *call, int retval )
 {
 //	int line_pad;
